"""
This type stub file was generated by pyright.
"""

import unittest

"""Experimental code for cleaner support of IPython syntax with unittest.

In IPython up until 0.10, we've used very hacked up nose machinery for running
tests with IPython special syntax, and this has proved to be extremely slow.
This module provides decorators to try a different approach, stemming from a
conversation Brian and I (FP) had about this problem Sept/09.

The goal is to be able to easily write simple functions that can be seen by
unittest as tests, and ultimately for these to support doctests with full
IPython syntax.  Nose already offers this based on naming conventions and our
hackish plugins, but we are seeking to move away from nose dependencies if
possible.

This module follows a different approach, based on decorators.

- A decorator called @ipdoctest can mark any function as having a docstring
  that should be viewed as a doctest, but after syntax conversion.

Authors
-------

- Fernando Perez <Fernando.Perez@berkeley.edu>
"""
def count_failures(runner):
    """Count number of failures in a doctest runner.

    Code modeled after the summarize() method in doctest.
    """
    ...

class IPython2PythonConverter:
    """Convert IPython 'syntax' to valid Python.

    Eventually this code may grow to be the full IPython syntax conversion
    implementation, but for now it only does prompt conversion."""
    def __init__(self) -> None:
        ...
    
    def __call__(self, ds):
        """Convert IPython prompts to python ones in a string."""
        ...
    


class Doc2UnitTester:
    """Class whose instances act as a decorator for docstring testing.

    In practice we're only likely to need one instance ever, made below (though
    no attempt is made at turning it into a singleton, there is no need for
    that).
    """
    def __init__(self, verbose=...) -> None:
        """New decorator.

        Parameters
        ----------

        verbose : boolean, optional (False)
          Passed to the doctest finder and runner to control verbosity.
        """
        ...
    
    def __call__(self, func): # -> Type[Tester]:
        """Use as a decorator: doctest a function's docstring as a unittest.
        
        This version runs normal doctests, but the idea is to make it later run
        ipython syntax instead."""
        class Tester(unittest.TestCase):
            ...
        
        
    


def ipdocstring(func):
    """Change the function docstring via ip2py.
    """
    ...

ipdoctest = ...
ip2py = ...



from typing import ClassVar, TypeVar
from scipy import linalg as linalg, optimize as optimize, sparse
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from numpy.random.mtrand import RandomState
from ..utils._array_api import get_namespace as get_namespace
from ..utils.sparsefuncs import (
    mean_variance_axis as mean_variance_axis,
    inplace_column_scale as inplace_column_scale,
)
from scipy.special import expit as expit
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils._seq_dataset import (
    ArrayDataset32 as ArrayDataset32,
    CSRDataset32 as CSRDataset32,
    ArrayDataset64,
    CSRDataset64,
)
from ..utils.validation import (
    FLOAT_DTYPES as FLOAT_DTYPES,
    check_is_fitted as check_is_fitted,
)
from scipy.sparse.linalg import lsqr as lsqr
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from numbers import Integral as Integral
from ..base import BaseEstimator, ClassifierMixin, RegressorMixin, MultiOutputMixin
from ..utils import check_array as check_array, check_random_state as check_random_state
from ._stochastic_gradient import SGDClassifier
from .._typing import MatrixLike, ArrayLike, Int

LinearRegression_Self = TypeVar("LinearRegression_Self", bound="LinearRegression")
SparseCoefMixin_Self = TypeVar("SparseCoefMixin_Self", bound="SparseCoefMixin")

import numbers
import warnings

import numpy as np
import scipy.sparse as sp

# TODO: bayesian_ridge_regression and bayesian_regression_ard
# should be squashed into its respective objects.

SPARSE_INTERCEPT_DECAY: float = ...


def make_dataset(
    X: MatrixLike,
    y: ArrayLike,
    sample_weight: ArrayLike,
    random_state: None | Int | RandomState = None,
) -> tuple[ArrayDataset64 | CSRDataset64, float]:
    ...


class LinearModel(BaseEstimator, metaclass=ABCMeta):
    @abstractmethod
    def fit(self, X, y):
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...


# XXX Should this derive from LinearModel? It should be a mixin, not an ABC.
# Maybe the n_features checking can be moved to LinearModel.
class LinearClassifierMixin(ClassifierMixin):
    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class SparseCoefMixin:
    def densify(self: SparseCoefMixin_Self) -> SparseCoefMixin_Self:
        ...

    def sparsify(self: SparseCoefMixin_Self) -> SGDClassifier | SparseCoefMixin_Self:
        ...


class LinearRegression(MultiOutputMixin, RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: float | ndarray = ...
    singular_: ndarray = ...
    rank_: int = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        fit_intercept: bool = True,
        copy_X: bool = True,
        n_jobs: None | Int = None,
        positive: bool = False,
    ) -> None:
        ...

    def fit(
        self: LinearRegression_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> LinearRegression_Self:
        ...



from typing import ClassVar, Literal, Mapping, Sequence, TypeVar
from numpy.random import RandomState
from scipy.sparse import issparse as issparse, spmatrix
from ..utils._bunch import Bunch
from ._criterion import Criterion as Criterion
from ..utils.validation import check_is_fitted as check_is_fitted
from abc import ABCMeta, abstractmethod
from ._splitter import Splitter as Splitter
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from ..base import (
    BaseEstimator,
    ClassifierMixin,
    clone as clone,
    RegressorMixin,
    is_classifier as is_classifier,
    MultiOutputMixin,
)
from ..utils import (
    check_random_state as check_random_state,
    compute_sample_weight as compute_sample_weight,
)
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from math import ceil as ceil
from .._typing import MatrixLike, ArrayLike, Int, Float
from ._tree import (
    DepthFirstTreeBuilder as DepthFirstTreeBuilder,
    BestFirstTreeBuilder as BestFirstTreeBuilder,
    Tree,
    ccp_pruning_path as ccp_pruning_path,
)

DecisionTreeRegressor_Self = TypeVar(
    "DecisionTreeRegressor_Self", bound="DecisionTreeRegressor"
)


# Authors: Gilles Louppe <g.louppe@gmail.com>
#          Peter Prettenhofer <peter.prettenhofer@gmail.com>
#          Brian Holt <bdholt1@gmail.com>
#          Noel Dawe <noel@dawe.me>
#          Satrajit Gosh <satrajit.ghosh@gmail.com>
#          Joly Arnaud <arnaud.v.joly@gmail.com>
#          Fares Hedayati <fares.hedayati@gmail.com>
#          Nelson Liu <nelson@nelsonliu.me>
#
# License: BSD 3 clause

import numbers
import warnings
import copy

import numpy as np

__all__ = [
    "DecisionTreeClassifier",
    "DecisionTreeRegressor",
    "ExtraTreeClassifier",
    "ExtraTreeRegressor",
]


# =============================================================================
# Types and constants
# =============================================================================

DTYPE = ...
DOUBLE = ...

CRITERIA_CLF: dict = ...
CRITERIA_REG: dict = ...

DENSE_SPLITTERS: dict = ...

SPARSE_SPLITTERS: dict = ...

# =============================================================================
# Base decision tree
# =============================================================================


class BaseDecisionTree(MultiOutputMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        *,
        criterion,
        splitter,
        max_depth,
        min_samples_split,
        min_samples_leaf,
        min_weight_fraction_leaf,
        max_features,
        max_leaf_nodes,
        random_state,
        min_impurity_decrease,
        class_weight=None,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...

    def get_depth(self) -> int:
        ...

    def get_n_leaves(self) -> int:
        ...

    def fit(
        self,
        X,
        y: ndarray,
        sample_weight: None | ndarray = None,
        check_input: bool = True,
    ):
        ...

    def predict(self, X: MatrixLike | ArrayLike, check_input: bool = True) -> ndarray:
        ...

    def apply(self, X: MatrixLike | ArrayLike, check_input: bool = True) -> ArrayLike:
        ...

    def decision_path(
        self, X: MatrixLike | ArrayLike, check_input: bool = True
    ) -> spmatrix:
        ...

    def cost_complexity_pruning_path(
        self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> Bunch:
        ...

    @property
    def feature_importances_(self) -> ndarray:
        ...


# =============================================================================
# Public estimators
# =============================================================================


class DecisionTreeClassifier(ClassifierMixin, BaseDecisionTree):
    tree_: Tree = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_classes_: int | list[int] = ...
    max_features_: int = ...
    feature_importances_: ndarray = ...
    classes_: ndarray | list[ndarray] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        criterion: Literal["gini", "entropy", "log_loss", "gini"] = "gini",
        splitter: Literal["best", "random", "best"] = "best",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | None | Literal["auto", "sqrt", "log2"] | int = None,
        random_state: RandomState | None | Int = None,
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        class_weight: None | Mapping | str | Sequence[Mapping] = None,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...

    def fit(
        self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        check_input: bool = True,
    ) -> DecisionTreeClassifier | ExtraTreeClassifier:
        ...

    def predict_proba(
        self, X: MatrixLike | ArrayLike, check_input: bool = True
    ) -> ndarray | list[ndarray]:
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray | list[ndarray]:
        ...


class DecisionTreeRegressor(RegressorMixin, BaseDecisionTree):
    tree_: Tree = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    max_features_: int = ...
    feature_importances_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        criterion: Literal[
            "squared_error",
            "friedman_mse",
            "absolute_error",
            "poisson",
            "squared_error",
        ] = "squared_error",
        splitter: Literal["best", "random", "best"] = "best",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | None | Literal["auto", "sqrt", "log2"] | int = None,
        random_state: RandomState | None | Int = None,
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...

    def fit(
        self: DecisionTreeRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        check_input: bool = True,
    ) -> DecisionTreeRegressor_Self:
        ...


class ExtraTreeClassifier(DecisionTreeClassifier):
    tree_: Tree = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    n_classes_: int | list[int] = ...
    max_features_: int = ...
    classes_: ndarray | list[ndarray] = ...

    def __init__(
        self,
        *,
        criterion: Literal["gini", "entropy", "log_loss", "gini"] = "gini",
        splitter: Literal["random", "best", "random"] = "random",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float
        | None
        | int
        | Literal["auto", "sqrt", "log2", "sqrt"] = "sqrt",
        random_state: RandomState | None | Int = None,
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        class_weight: None | Mapping | str | Sequence[Mapping] = None,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...


class ExtraTreeRegressor(DecisionTreeRegressor):
    tree_: Tree = ...
    n_outputs_: int = ...
    feature_importances_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    max_features_: int = ...

    def __init__(
        self,
        *,
        criterion: Literal[
            "squared_error",
            "friedman_mse",
            "absolute_error",
            "poisson",
            "squared_error",
        ] = "squared_error",
        splitter: Literal["random", "best", "random"] = "random",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | None | Literal["auto", "sqrt", "log2"] | int = 1.0,
        random_state: RandomState | None | Int = None,
        min_impurity_decrease: Float = 0.0,
        max_leaf_nodes: None | Int = None,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...



import os
from collections import OrderedDict
from collections.abc import Callable, Iterator, KeysView, ValuesView
from typing import Any, TypeVar

from transformers.configuration_utils import PretrainedConfig

_F = TypeVar('_F', bound=Callable[..., Any])

CONFIG_MAPPING_NAMES: OrderedDict[str, str]
CONFIG_ARCHIVE_MAP_MAPPING_NAMES: OrderedDict[str, str]
MODEL_NAMES_MAPPING: OrderedDict[str, str]
SPECIAL_MODEL_TYPE_TO_MODULE_NAME: OrderedDict[str, str]

def model_type_to_module_name(key) -> str: ...
def config_class_to_model_type(config) -> str | None: ...

class _LazyConfigMapping(OrderedDict[str, type[PretrainedConfig]]):
    def __init__(self, mapping) -> None: ...
    def __getitem__(self, key: str) -> type[PretrainedConfig]: ...
    def keys(self) -> list[str]: ...
    def values(self) -> list[type[PretrainedConfig]]: ...
    def items(self) -> list[tuple[str, type[PretrainedConfig]]]: ...
    def __iter__(self) -> Iterator[str]: ...
    def __contains__(self, item: object) -> bool: ...
    def register(self, key: str, value: type[PretrainedConfig]) -> None: ...

CONFIG_MAPPING: _LazyConfigMapping

class _LazyLoadAllMappings(OrderedDict[str, str]):
    def __init__(self, mapping) -> None: ...
    def __getitem__(self, key: str) -> str: ...
    def keys(self) -> KeysView[str]: ...
    def values(self) -> ValuesView[str]: ...
    def items(self) -> KeysView[str]: ...
    def __iter__(self) -> Iterator[str]: ...
    def __contains__(self, item: object) -> bool: ...

ALL_PRETRAINED_CONFIG_ARCHIVE_MAP: _LazyLoadAllMappings

def replace_list_option_in_docstrings(config_to_class=None, use_model_types: bool = True) -> Callable[[_F], _F]: ...

class AutoConfig:
    def __init__(self) -> None: ...
    @classmethod
    def for_model(cls, model_type: str, *args, **kwargs) -> PretrainedConfig: ...
    @classmethod
    def from_pretrained(cls, pretrained_model_name_or_path: str | os.PathLike[str], **kwargs): ...
    @staticmethod
    def register(model_type, config) -> None: ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import sys

from ._vispy_fonts import _vispy_fonts

_fonts: dict = ...

def list_fonts() -> ArrayLike: ...



from matplotlib.ft2font import FT2Font
from typing import Literal
from ._typing import *
from ._afm import AFM

from pathlib import PosixPath

import json
from functools import lru_cache

font_scalings = ...
stretch_dict = ...
weight_dict = ...

font_family_aliases = ...
MSFolders = ...
MSFontDirectories = ...
MSUserFontDirectories = ...
X11FontDirectories = ...
OSXFontDirectories = ...

def get_fontext_synonyms(fontext): ...
def list_fonts(directory, extensions): ...
def win32FontDirectory(): ...
def win32InstalledFonts(directory=..., fontext=...): ...
def get_fontconfig_fonts(fontext=...): ...
def findSystemFonts(fontpaths=..., fontext=...): ...

FontEntry = ...

def ttfFontProperty(font: FT2Font) -> FontEntry: ...
def afmFontProperty(fontpath, font: AFM) -> FontEntry: ...

class FontProperties:
    def __init__(
        self,
        family: list[str]
        | str
        | Literal["sans-serif", "serif", "cursive", "fantasy", "monospace"] = ...,
        style: Literal["normal", "italic", "oblique"] = ...,
        variant: Literal["normal", "small-caps"] = ...,
        weight: int
        | Literal[
            "ultralight",
            "light",
            "normal",
            "regular",
            "book",
            "medium",
            "roman",
            "semibold",
            "demibold",
            "demi",
            "bold",
            "heavy",
            "extra bold",
            "black",
        ] = ...,
        stretch: int
        | Literal[
            "ultra-condensed",
            "extra-condensed",
            "condensed",
            "semi-condensed",
            "normal",
            "semi-expanded",
            "expanded",
            "extra-expanded",
            "ultra-expanded",
        ] = ...,
        size: float
        | Literal[
            "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"
        ] = ...,
        fname: PosixPath | None = ...,
        math_fontfamily: Literal[
            "dejavusans", "dejavuserif", "cm", "stix", "stixsans", "custom"
        ] = ...,
    ) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: "FontProperties") -> bool: ...
    def __str__(self) -> str: ...
    def get_family(self) -> list[str]: ...
    def get_name(self) -> str: ...
    def get_style(self) -> str: ...
    def get_variant(self) -> str: ...
    def get_weight(self) -> str: ...
    def get_stretch(self) -> str: ...
    def get_size(self) -> float: ...
    def get_file(self) -> str | None: ...
    def get_fontconfig_pattern(self) -> str: ...
    def set_family(
        self,
        family: str
        | list[str]
        | Literal["sans-serif", "serif", "cursive", "fantasy", "monospace"],
    ) -> None: ...
    def set_style(self, style: Literal["normal", "italic", "oblique"]) -> None: ...
    def set_variant(self, variant: Literal["normal", "small-caps"]) -> None: ...
    def set_weight(
        self,
        weight: int
        | Literal[
            "ultralight",
            "light",
            "normal",
            "regular",
            "book",
            "medium",
            "roman",
            "semibold",
            "demibold",
            "demi",
            "bold",
            "heavy",
            "extra bold",
            "black",
        ],
    ) -> None: ...
    def set_stretch(
        self,
        stretch: int
        | Literal[
            "ultra-condensed",
            "extra-condensed",
            "condensed",
            "semi-condensed",
            "normal",
            "semi-expanded",
            "expanded",
            "extra-expanded",
            "ultra-expanded",
        ],
    ) -> None: ...
    def set_size(
        self,
        size: float
        | Literal[
            "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"
        ],
    ) -> None: ...
    def set_file(self, file: PosixPath | None) -> None: ...
    def set_fontconfig_pattern(self, pattern: str) -> None: ...
    def get_math_fontfamily(self) -> str: ...
    def set_math_fontfamily(
        self,
        fontfamily: Literal[
            "dejavusans", "dejavuserif", "cm", "stix", "stixsans", "custom"
        ],
    ) -> None: ...
    def copy(self) -> "FontProperties": ...
    set_name = ...
    get_slant = ...
    set_slant = ...
    get_size_in_points = ...

class _JSONEncoder(json.JSONEncoder):
    def default(self, o): ...

def json_dump(data, filename): ...
def json_load(filename: PosixPath) -> FontManager: ...

class FontManager:
    def __init__(self, size=..., weight=...) -> None: ...
    def addfont(self, path: str | PathLike): ...
    @property
    def defaultFont(self): ...
    def get_default_weight(self): ...
    @staticmethod
    def get_default_size() -> float: ...
    def set_default_weight(
        self,
        weight: int
        | Literal[
            "ultralight",
            "light",
            "normal",
            "regular",
            "book",
            "medium",
            "roman",
            "semibold",
            "demibold",
            "demi",
            "bold",
            "heavy",
            "extra bold",
            "black",
        ],
    ): ...
    def score_family(self, families: list[str], family2: str) -> float: ...
    def score_style(self, style1: str, style2: str) -> float: ...
    def score_variant(self, variant1: str, variant2: str) -> float: ...
    def score_stretch(self, stretch1: str, stretch2: str) -> float: ...
    def score_weight(self, weight1: str, weight2: int) -> float: ...
    def score_size(self, size1: float, size2: str) -> float: ...
    def findfont(
        self,
        prop: FontProperties | str,
        fontext: Literal["ttf", "afm"] = ...,
        directory: str = ...,
        fallback_to_default: bool = ...,
        rebuild_if_missing: bool = ...,
    ) -> str: ...
    def get_font_names(self) -> list: ...

@lru_cache()
def is_opentype_cff_font(filename): ...
def get_font(filename, hinting_factor=...): ...

fontManager = ...
findfont = ...
get_font_names = ...



from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy.special import gammaln as gammaln, logsumexp as logsumexp
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from joblib import effective_n_jobs as effective_n_jobs
from numbers import Integral as Integral, Real as Real
from ._online_lda_fast import mean_change as cy_mean_change
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import Int, Float, MatrixLike, ArrayLike
from ..utils import (
    check_random_state as check_random_state,
    gen_batches as gen_batches,
    gen_even_slices as gen_even_slices,
)
from ..utils.validation import (
    check_non_negative as check_non_negative,
    check_is_fitted as check_is_fitted,
)

LatentDirichletAllocation_Self = TypeVar(
    "LatentDirichletAllocation_Self", bound="LatentDirichletAllocation"
)


import numpy as np
import scipy.sparse as sp

EPS = ...


class LatentDirichletAllocation(
    ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator
):
    topic_word_prior_: float = ...
    random_state_: RandomState = ...
    doc_topic_prior_: float = ...
    bound_: float = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_batch_iter_: int = ...
    exp_dirichlet_component_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: Int = 10,
        *,
        doc_topic_prior: None | Float = None,
        topic_word_prior: None | Float = None,
        learning_method: Literal["batch", "online", "batch"] = "batch",
        learning_decay: Float = 0.7,
        learning_offset: Float = 10.0,
        max_iter: Int = 10,
        batch_size: Int = 128,
        evaluate_every: Int = ...,
        total_samples: float | Int = 1e6,
        perp_tol: Float = 1e-1,
        mean_change_tol: Float = 1e-3,
        max_doc_update_iter: Int = 100,
        n_jobs: None | Int = None,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def partial_fit(
        self: LatentDirichletAllocation_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> LatentDirichletAllocation_Self:
        ...

    def fit(
        self: LatentDirichletAllocation_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> LatentDirichletAllocation_Self:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def score(self, X: MatrixLike | ArrayLike, y: Any = None) -> float:
        ...

    def perplexity(
        self, X: MatrixLike | ArrayLike, sub_sampling: bool = False
    ) -> float:
        ...



from typing import Mapping

# This module uses material from the Wikipedia article Hopcroft--Karp algorithm
# <https://en.wikipedia.org/wiki/Hopcroft%E2%80%93Karp_algorithm>, accessed on
# January 3, 2015, which is released under the Creative Commons
# Attribution-Share-Alike License 3.0
# <http://creativecommons.org/licenses/by-sa/3.0/>. That article includes
# pseudocode, which has been translated into the corresponding Python code.
#
# Portions of this module use code from David Eppstein's Python Algorithms and
# Data Structures (PADS) library, which is dedicated to the public domain (for
# proof, see <http://www.ics.uci.edu/~eppstein/PADS/ABOUT-PADS.txt>).
import collections
import itertools

from .matrix import biadjacency_matrix
from ...classes.graph import Graph

__all__ = [
    "maximum_matching",
    "hopcroft_karp_matching",
    "eppstein_matching",
    "to_vertex_cover",
    "minimum_weight_full_matching",
]

INFINITY = ...

def hopcroft_karp_matching(G: Graph, top_nodes=None) -> Mapping: ...
def eppstein_matching(G: Graph, top_nodes=None) -> Mapping: ...
def to_vertex_cover(G: Graph, matching: Mapping, top_nodes=None): ...

#: Returns the maximum cardinality matching in the given bipartite graph.
#:
#: This function is simply an alias for :func:`hopcroft_karp_matching`.
maximum_matching = ...

def minimum_weight_full_matching(
    G: Graph, top_nodes=None, weight="weight"
) -> Mapping: ...



"""Bump template `.pre-commit-config.yaml` and `requirements.txt`.

Synchronize `black` and `ruff` in pre-commit and `requirements.txt`.
"""

import re
from pathlib import Path

PRE_COMMIT = Path("template/.pre-commit-config.yaml")
REQUIREMENTS = Path("template/.tools/requirements/requirements_both.txt")

patterns = {
    "black": r"""
  - repo: "https:\/\/github\.com\/psf\/black"
    rev: "v?(?P<version>[\d.]+)"
""",
    "ruff": r"""
  - repo: "https:\/\/github\.com\/charliermarsh\/ruff-pre-commit"
    rev: "v?(?P<version>[\d.]+)"
""",
}

# Bump pre-commit
pre_commit_text = PRE_COMMIT.read_text(encoding="utf-8")
versions = {
    package: re.search(pattern, pre_commit_text)["version"]  # pyright: ignore
    for package, pattern in patterns.items()
}
# fmt: off
# Visually align these long lines before mapping them below
black_find =  '        additional_dependencies: ["black=='
black_repl = f'        additional_dependencies: ["black=={versions["black"]}"]'
ruff_find =   '        additional_dependencies: ["ruff=='
ruff_repl =  f'        additional_dependencies: ["ruff=={versions["ruff"]}"]'
# fmt: on
pre_commit_replace = {black_find: black_repl, ruff_find: ruff_repl}
pre_commit_lines = pre_commit_text.split("\n")
for line_number, line in enumerate(pre_commit_lines):
    if line.startswith(black_find):
        pre_commit_lines[line_number] = black_repl
    elif line.startswith(ruff_find):
        pre_commit_lines[line_number] = ruff_repl
PRE_COMMIT.write_text(encoding="utf-8", data="\n".join(pre_commit_lines))

# Bump requirements
requirements_lines = REQUIREMENTS.read_text(encoding="utf-8").split("\n")
for line_number, line in enumerate(requirements_lines):
    if line.startswith("black"):
        requirements_lines[line_number] = f"black=={versions['black']}"
    if line.startswith("ruff"):
        requirements_lines[line_number] = f"ruff=={versions['ruff']}"
REQUIREMENTS.write_text(encoding="utf-8", data="\n".join(requirements_lines))



from itertools import chain
from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = [
    "biconnected_components",
    "biconnected_component_edges",
    "is_biconnected",
    "articulation_points",
]

def is_biconnected(G: Graph) -> bool: ...
def biconnected_component_edges(G: Graph): ...
def biconnected_components(G: Graph): ...
def articulation_points(G: Graph): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import re

# regular expressions for parsing GLSL

re_version_pragma: str = ...

re_type: str = ...
re_identifier: str = ...

# variable qualifiers
re_qualifier: str = ...

# template variables like
#     $func_name
re_template_var = ...

# function names may be either identifier or template var
re_func_name = ...

# type and identifier like "vec4 var_name"
re_declaration = ...

# qualifier, type, and identifier like "uniform vec4 var_name"
# qualifier is optional.
# may include multiple names like "attribute float x, y, z"
re_prog_var_declaration = ...

# list of variable declarations like "vec4 var_name, float other_var_name"
re_arg_list = ...

# function declaration like "vec4 function_name(float x, float y)"
re_func_decl = ...

# anonymous variable declarations may or may not include a name:
#  "vec4" or "vec4 var_name"
re_anon_decl = ...

# list of anonymous declarations
re_anon_arg_list = ...

# function prototype declaration like
#    "vec4 function_name(float, float);"
re_func_prot = ...

def parse_function_signature(code): ...
def find_functions(code): ...
def find_prototypes(code): ...
def find_program_variables(code): ...
def find_template_variables(code): ...



from typing import Any, ClassVar, Literal, TypeVar
from warnings import warn as warn
from numpy.random import RandomState
from ..tree._tree import DTYPE as tree_dtype
from ..tree import ExtraTreeRegressor
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ._bagging import BaseBagging
from scipy.sparse import issparse as issparse
from ..base import OutlierMixin
from .._typing import Int, MatrixLike, ArrayLike
from ..utils import (
    check_random_state as check_random_state,
    check_array as check_array,
    gen_batches as gen_batches,
    get_chunk_n_rows as get_chunk_n_rows,
)
from ..utils.validation import check_is_fitted as check_is_fitted

IsolationForest_Self = TypeVar("IsolationForest_Self", bound="IsolationForest")

# Authors: Nicolas Goix <nicolas.goix@telecom-paristech.fr>
#          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
# License: BSD 3 clause

import numbers
import numpy as np

__all__ = ["IsolationForest"]


class IsolationForest(OutlierMixin, BaseBagging):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    offset_: float = ...
    max_samples_: int = ...
    estimators_samples_: list[ndarray] = ...
    estimators_features_: list[ndarray] = ...
    estimators_: list[ExtraTreeRegressor] = ...
    base_estimator_: ExtraTreeRegressor = ...
    estimator_: ExtraTreeRegressor = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_estimators: Int = 100,
        max_samples: float | Literal["auto", "auto"] | int = "auto",
        contamination: float | str = "auto",
        max_features: float | int = 1.0,
        bootstrap: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        warm_start: bool = False,
    ) -> None:
        ...

    def fit(
        self: IsolationForest_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> IsolationForest_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def score_samples(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



import math
import numpy as np

from . import polygon as draw_polygon, disk as draw_disk, ellipse as draw_ellipse
from .._shared.utils import deprecate_multichannel_kwarg, warn

def _generate_rectangle_mask(point, image, shape, random): ...
def _generate_circle_mask(point, image, shape, random): ...
def _generate_triangle_mask(point, image, shape, random): ...
def _generate_ellipse_mask(point, image, shape, random): ...

# Allows lookup by key as well as random selection.
SHAPE_GENERATORS = ...
SHAPE_CHOICES = ...

def _generate_random_colors(num_colors, num_channels, intensity_range, random): ...
@deprecate_multichannel_kwarg(multichannel_position=5)
def random_shapes(
    image_shape: tuple,
    max_shapes: int,
    min_shapes: int = 1,
    min_size: int = 2,
    max_size: int | None = None,
    multichannel: bool = True,
    num_channels: int = 3,
    shape=None,
    intensity_range=None,
    allow_overlap: bool = False,
    num_trials: int = 100,
    random_seed=None,
    *,
    channel_axis: int | None = ...
): ...



from .backend_bases import RendererBase
import functools

class TexManager:

    texcache = ...

    grey_arrayd = ...
    font_family = ...
    font_families = ...
    font_info = ...
    @functools.lru_cache()
    def __new__(cls) -> TexManager: ...
    def get_font_config(self): ...
    @classmethod
    def get_basefile(cls, tex: str, fontsize: float, dpi: None = ...) -> str: ...
    @classmethod
    def get_font_preamble(cls): ...
    @classmethod
    def get_custom_preamble(cls) -> str: ...
    @classmethod
    def make_tex(cls, tex: str, fontsize: float) -> str: ...
    @classmethod
    def make_dvi(cls, tex: str, fontsize: float): ...
    @classmethod
    def make_png(cls, tex: str, fontsize: float, dpi: float): ...
    @classmethod
    def get_grey(cls, tex: str, fontsize: float = ..., dpi: float = ...): ...
    @classmethod
    def get_rgba(cls, tex: str, fontsize: float = ..., dpi: float = ..., rgb=...): ...
    @classmethod
    def get_text_width_height_descent(
        cls, tex: str, fontsize: float, renderer: RendererBase = ...
    ): ...



"""
This type stub file was generated by pyright.
"""

import importlib.abc

"""
This module contains factory functions that attempt
to return Qt submodules from the various python Qt bindings.

It also protects against double-importing Qt with different
bindings, which is unstable and likely to crash

This is used primarily by qt and qt_for_kernel, and shouldn't
be accessed directly from the outside
"""
QT_API_PYQT6 = ...
QT_API_PYSIDE6 = ...
QT_API_PYQT5 = ...
QT_API_PYSIDE2 = ...
QT_API_PYQT = ...
QT_API_PYQTv1 = ...
QT_API_PYSIDE = ...
QT_API_PYQT_DEFAULT = ...
api_to_module = ...
class ImportDenier(importlib.abc.MetaPathFinder):
    """Import Hook that will guard against bad Qt imports
    once IPython commits to a specific binding
    """
    def __init__(self) -> None:
        ...
    
    def forbid(self, module_name): # -> None:
        ...
    
    def find_spec(self, fullname, path, target=...): # -> None:
        ...
    


ID = ...
def commit_api(api): # -> None:
    """Commit to a particular API, and trigger ImportErrors on subsequent
    dangerous imports"""
    ...

def loaded_api(): # -> None:
    """Return which API is loaded, if any

    If this returns anything besides None,
    importing any other Qt binding is unsafe.

    Returns
    -------
    None, 'pyside6', 'pyqt6', 'pyside2', 'pyside', 'pyqt', 'pyqt5', 'pyqtv1'
    """
    ...

def has_binding(api):
    """Safely check for PyQt4/5, PySide or PySide2, without importing submodules

    Parameters
    ----------
    api : str [ 'pyqtv1' | 'pyqt' | 'pyqt5' | 'pyside' | 'pyside2' | 'pyqtdefault']
        Which module to check for

    Returns
    -------
    True if the relevant module appears to be importable
    """
    ...

def qtapi_version(): # -> None:
    """Return which QString API has been set, if any

    Returns
    -------
    The QString API version (1 or 2), or None if not set
    """
    ...

def can_import(api):
    """Safely query whether an API is importable, without importing it"""
    ...

def import_pyqt4(version=...):
    """
    Import PyQt4

    Parameters
    ----------
    version : 1, 2, or None
        Which QString/QVariant API to use. Set to None to use the system
        default
    ImportErrors raised within this function are non-recoverable
    """
    ...

def import_pyqt5():
    """
    Import PyQt5

    ImportErrors raised within this function are non-recoverable
    """
    ...

def import_pyqt6():
    """
    Import PyQt6

    ImportErrors raised within this function are non-recoverable
    """
    ...

def import_pyside():
    """
    Import PySide

    ImportErrors raised within this function are non-recoverable
    """
    ...

def import_pyside2():
    """
    Import PySide2

    ImportErrors raised within this function are non-recoverable
    """
    ...

def import_pyside6():
    """
    Import PySide6

    ImportErrors raised within this function are non-recoverable
    """
    ...

def load_qt(api_options):
    """
    Attempt to import Qt, given a preference list
    of permissible bindings

    It is safe to call this function multiple times.

    Parameters
    ----------
    api_options : List of strings
        The order of APIs to try. Valid items are 'pyside', 'pyside2',
        'pyqt', 'pyqt5', 'pyqtv1' and 'pyqtdefault'

    Returns
    -------
    A tuple of QtCore, QtGui, QtSvg, QT_API
    The first three are the Qt modules. The last is the
    string indicating which module was loaded.

    Raises
    ------
    ImportError, if it isn't possible to import any requested
    bindings (either because they aren't installed, or because
    an incompatible library has already been installed)
    """
    ...

def enum_factory(QT_API, QtCore): # -> (name: Unknown) -> Unknown:
    """Construct an enum helper to account for PyQt5 <-> PyQt6 changes."""
    ...




# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# Based on the quaternion class in the visvis project.

import numpy as np

class Quaternion(object):
    def __init__(self, w=1, x=0, y=0, z=0, normalize=True): ...
    def __repr__(self): ...
    def copy(self): ...
    def norm(self): ...
    def _normalize(self): ...
    def normalize(self): ...
    def conjugate(self): ...
    def inverse(self): ...
    def exp(self): ...
    def log(self): ...
    def __add__(self, q): ...
    def __sub__(self, q): ...
    def __mul__(self, q2): ...
    def rotate_point(self, p): ...
    def get_matrix(self): ...
    def get_axis_angle(self): ...
    @classmethod
    def create_from_axis_angle(cls, angle, ax, ay, az, degrees=False): ...
    @classmethod
    def create_from_euler_angles(cls, rx, ry, rz, degrees=False): ...



# -*- coding: utf-8 -*-
# This file was auto-generated from `mc_meta/LookUpTable.h` by
# `mc_meta/createluts.py`.

# static const char casesClassic[256][16]
CASESCLASSIC = ...

# static const char cases[256][2]
CASES = ...

# static const char tiling1[16][3]
TILING1 = ...

# static const char tiling2[24][6]
TILING2 = ...

# static const char tiling3_1[24][6]
TILING3_1 = ...

# static const char tiling3_2[24][12]
TILING3_2 = ...

# static const char tiling4_1[8][6]
TILING4_1 = ...

# static const char tiling4_2[8][18]
TILING4_2 = ...

# static const char tiling5[48][9]
TILING5 = ...

# static const char tiling6_1_1[48][9]
TILING6_1_1 = ...

# static const char tiling6_1_2[48][27]
TILING6_1_2 = ...

# static const char tiling6_2[48][15]
TILING6_2 = ...

# static const char tiling7_1[16][9]
TILING7_1 = ...

# static const char tiling7_2[16][3][15]
TILING7_2 = ...

# static const char tiling7_3[16][3][27]
TILING7_3 = ...

# static const char tiling7_4_1[16][15]
TILING7_4_1 = ...

# static const char tiling7_4_2[16][27]
TILING7_4_2 = ...

# static const char tiling8[6][6]
TILING8 = ...

# static const char tiling9[8][12]
TILING9 = ...

# static const char tiling10_1_1[6][12]
TILING10_1_1 = ...

# static const char tiling10_1_1_[6][12]
TILING10_1_1_ = ...

# static const char tiling10_1_2[6][24]
TILING10_1_2 = ...

# static const char tiling10_2[6][24]
TILING10_2 = ...

# static const char tiling10_2_[6][24]
TILING10_2_ = ...

# static const char tiling11[12][12]
TILING11 = ...

# static const char tiling12_1_1[24][12]
TILING12_1_1 = ...

# static const char tiling12_1_1_[24][12]
TILING12_1_1_ = ...

# static const char tiling12_1_2[24][24]
TILING12_1_2 = ...

# static const char tiling12_2[24][24]
TILING12_2 = ...

# static const char tiling12_2_[24][24]
TILING12_2_ = ...

# static const char tiling13_1[2][12]
TILING13_1 = ...

# static const char tiling13_1_[2][12]
TILING13_1_ = ...

# static const char tiling13_2[2][6][18]
TILING13_2 = ...

# static const char tiling13_2_[2][6][18]
TILING13_2_ = ...

# static const char tiling13_3[2][12][30]
TILING13_3 = ...

# static const char tiling13_3_[2][12][30]
TILING13_3_ = ...

# static const char tiling13_4[2][4][36]
TILING13_4 = ...

# static const char tiling13_5_1[2][4][18]
TILING13_5_1 = ...

# static const char tiling13_5_2[2][4][30]
TILING13_5_2 = ...

# static const char tiling14[12][12]
TILING14 = ...

# static const char test3[24]
TEST3 = ...

# static const char test4[8]
TEST4 = ...

# static const char test6[48][3]
TEST6 = ...

# static const char test7[16][5]
TEST7 = ...

# static const char test10[6][3]
TEST10 = ...

# static const char test12[24][4]
TEST12 = ...

# static const char test13[2][7]
TEST13 = ...

# static const char subconfig13[64]
SUBCONFIG13 = ...



from typing import Mapping
from numpy.typing import ArrayLike

from collections import deque
from heapq import heappop, heappush
from itertools import count

from ...classes.graph import Graph

__all__ = [
    "dijkstra_path",
    "dijkstra_path_length",
    "bidirectional_dijkstra",
    "single_source_dijkstra",
    "single_source_dijkstra_path",
    "single_source_dijkstra_path_length",
    "multi_source_dijkstra",
    "multi_source_dijkstra_path",
    "multi_source_dijkstra_path_length",
    "all_pairs_dijkstra",
    "all_pairs_dijkstra_path",
    "all_pairs_dijkstra_path_length",
    "dijkstra_predecessor_and_distance",
    "bellman_ford_path",
    "bellman_ford_path_length",
    "single_source_bellman_ford",
    "single_source_bellman_ford_path",
    "single_source_bellman_ford_path_length",
    "all_pairs_bellman_ford_path",
    "all_pairs_bellman_ford_path_length",
    "bellman_ford_predecessor_and_distance",
    "negative_edge_cycle",
    "find_negative_cycle",
    "goldberg_radzik",
    "johnson",
]

def dijkstra_path(G: Graph, source, target, weight="weight") -> ArrayLike: ...
def dijkstra_path_length(G: Graph, source, target, weight="weight"): ...
def single_source_dijkstra_path(
    G: Graph, source, cutoff=None, weight="weight"
) -> Mapping: ...
def single_source_dijkstra_path_length(
    G: Graph, source, cutoff=None, weight="weight"
) -> Mapping: ...
def single_source_dijkstra(
    G: Graph, source, target=None, cutoff=None, weight="weight"
): ...
def multi_source_dijkstra_path(
    G: Graph, sources, cutoff=None, weight="weight"
) -> Mapping: ...
def multi_source_dijkstra_path_length(
    G: Graph, sources, cutoff=None, weight="weight"
) -> Mapping: ...
def multi_source_dijkstra(
    G: Graph, sources, target=None, cutoff=None, weight="weight"
): ...
def dijkstra_predecessor_and_distance(
    G: Graph, source, cutoff=None, weight="weight"
) -> dict: ...
def all_pairs_dijkstra(G: Graph, cutoff=None, weight="weight"): ...
def all_pairs_dijkstra_path_length(G: Graph, cutoff=None, weight="weight"): ...
def all_pairs_dijkstra_path(G: Graph, cutoff=None, weight="weight") -> Mapping: ...
def bellman_ford_predecessor_and_distance(
    G: Graph, source, target=None, weight="weight", heuristic: bool = False
) -> dict: ...
def bellman_ford_path(G: Graph, source, target, weight="weight") -> ArrayLike: ...
def bellman_ford_path_length(G: Graph, source, target, weight="weight"): ...
def single_source_bellman_ford_path(G: Graph, source, weight="weight") -> Mapping: ...
def single_source_bellman_ford_path_length(G: Graph, source, weight="weight"): ...
def single_source_bellman_ford(G: Graph, source, target=None, weight="weight"): ...
def all_pairs_bellman_ford_path_length(G: Graph, weight="weight"): ...
def all_pairs_bellman_ford_path(G: Graph, weight="weight") -> Mapping: ...
def goldberg_radzik(G: Graph, source, weight="weight") -> dict: ...
def negative_edge_cycle(G: Graph, weight="weight", heuristic: bool = True) -> bool: ...
def find_negative_cycle(G: Graph, source, weight="weight") -> ArrayLike: ...
def bidirectional_dijkstra(G: Graph, source, target, weight="weight"): ...
def johnson(G: Graph, weight="weight") -> Mapping: ...



from .graph_cut import cut_threshold as cut_threshold, cut_normalized as cut_normalized
from .rag import (
    rag_mean_color as rag_mean_color,
    RAG as RAG,
    show_rag as show_rag,
    rag_boundary as rag_boundary,
)
from .graph_merge import merge_hierarchical as merge_hierarchical

ncut = ...

__all__ = [
    "rag_mean_color",
    "cut_threshold",
    "cut_normalized",
    "ncut",
    "show_rag",
    "merge_hierarchical",
    "rag_boundary",
    "RAG",
]



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.spatial.ckdtree, version: unspecified
import typing
import builtins as _mod_builtins

__all__: list
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

def __pyx_unpickle_cKDTreeNode() -> typing.Any:
    ...

__test__: dict
class cKDTree(_mod_builtins.object):
    '\n    cKDTree(data, leafsize=16, compact_nodes=True, copy_data=False,\n            balanced_tree=True, boxsize=None)\n\n    kd-tree for quick nearest-neighbor lookup\n\n    This class provides an index into a set of k-dimensional points\n    which can be used to rapidly look up the nearest neighbors of any\n    point.\n\n    Parameters\n    ----------\n    data : array_like, shape (n,m)\n        The n data points of dimension m to be indexed. This array is\n        not copied unless this is necessary to produce a contiguous\n        array of doubles, and so modifying this data will result in\n        bogus results. The data are also copied if the kd-tree is built\n        with copy_data=True.\n    leafsize : positive int, optional\n        The number of points at which the algorithm switches over to\n        brute-force. Default: 16.\n    compact_nodes : bool, optional\n        If True, the kd-tree is built to shrink the hyperrectangles to\n        the actual data range. This usually gives a more compact tree that\n        is robust against degenerated input data and gives faster queries\n        at the expense of longer build time. Default: True.\n    copy_data : bool, optional\n        If True the data is always copied to protect the kd-tree against\n        data corruption. Default: False.\n    balanced_tree : bool, optional\n        If True, the median is used to split the hyperrectangles instead of\n        the midpoint. This usually gives a more compact tree and\n        faster queries at the expense of longer build time. Default: True.\n    boxsize : array_like or scalar, optional\n        Apply a m-d toroidal topology to the KDTree.. The topology is generated\n        by :math:`x_i + n_i L_i` where :math:`n_i` are integers and :math:`L_i`\n        is the boxsize along i-th dimension. The input data shall be wrapped\n        into :math:`[0, L_i)`. A ValueError is raised if any of the data is\n        outside of this bound.\n\n    Notes\n    -----\n    The algorithm used is described in Maneewongvatana and Mount 1999.\n    The general idea is that the kd-tree is a binary tree, each of whose\n    nodes represents an axis-aligned hyperrectangle. Each node specifies\n    an axis and splits the set of points based on whether their coordinate\n    along that axis is greater than or less than a particular value.\n\n    During construction, the axis and splitting point are chosen by the\n    "sliding midpoint" rule, which ensures that the cells do not all\n    become long and thin.\n\n    The tree can be queried for the r closest neighbors of any given point\n    (optionally returning only those within some maximum distance of the\n    point). It can also be queried, with a substantial gain in efficiency,\n    for the r approximate closest neighbors.\n\n    For large dimensions (20 is already large) do not expect this to run\n    significantly faster than brute force. High-dimensional nearest-neighbor\n    queries are a substantial open problem in computer science.\n\n    Attributes\n    ----------\n    data : ndarray, shape (n,m)\n        The n data points of dimension m to be indexed. This array is\n        not copied unless this is necessary to produce a contiguous\n        array of doubles. The data are also copied if the kd-tree is built\n        with `copy_data=True`.\n    leafsize : positive int\n        The number of points at which the algorithm switches over to\n        brute-force.\n    m : int\n        The dimension of a single data-point.\n    n : int\n        The number of data points.\n    maxes : ndarray, shape (m,)\n        The maximum value in each dimension of the n data points.\n    mins : ndarray, shape (m,)\n        The minimum value in each dimension of the n data points.\n    tree : object, class cKDTreeNode\n        This attribute exposes a Python view of the root node in the cKDTree\n        object. A full Python view of the kd-tree is created dynamically\n        on the first access. This attribute allows you to create your own\n        query functions in Python.\n    size : int\n        The number of nodes in the tree.\n\n    '
    def __getstate__(self) -> typing.Any:
        ...
    
    def __init__(self, data, leafsize=..., compact_nodes=..., copy_data=..., balanced_tree=..., boxsize=...) -> None:
        '\n    cKDTree(data, leafsize=16, compact_nodes=True, copy_data=False,\n            balanced_tree=True, boxsize=None)\n\n    kd-tree for quick nearest-neighbor lookup\n\n    This class provides an index into a set of k-dimensional points\n    which can be used to rapidly look up the nearest neighbors of any\n    point.\n\n    Parameters\n    ----------\n    data : array_like, shape (n,m)\n        The n data points of dimension m to be indexed. This array is\n        not copied unless this is necessary to produce a contiguous\n        array of doubles, and so modifying this data will result in\n        bogus results. The data are also copied if the kd-tree is built\n        with copy_data=True.\n    leafsize : positive int, optional\n        The number of points at which the algorithm switches over to\n        brute-force. Default: 16.\n    compact_nodes : bool, optional\n        If True, the kd-tree is built to shrink the hyperrectangles to\n        the actual data range. This usually gives a more compact tree that\n        is robust against degenerated input data and gives faster queries\n        at the expense of longer build time. Default: True.\n    copy_data : bool, optional\n        If True the data is always copied to protect the kd-tree against\n        data corruption. Default: False.\n    balanced_tree : bool, optional\n        If True, the median is used to split the hyperrectangles instead of\n        the midpoint. This usually gives a more compact tree and\n        faster queries at the expense of longer build time. Default: True.\n    boxsize : array_like or scalar, optional\n        Apply a m-d toroidal topology to the KDTree.. The topology is generated\n        by :math:`x_i + n_i L_i` where :math:`n_i` are integers and :math:`L_i`\n        is the boxsize along i-th dimension. The input data shall be wrapped\n        into :math:`[0, L_i)`. A ValueError is raised if any of the data is\n        outside of this bound.\n\n    Notes\n    -----\n    The algorithm used is described in Maneewongvatana and Mount 1999.\n    The general idea is that the kd-tree is a binary tree, each of whose\n    nodes represents an axis-aligned hyperrectangle. Each node specifies\n    an axis and splits the set of points based on whether their coordinate\n    along that axis is greater than or less than a particular value.\n\n    During construction, the axis and splitting point are chosen by the\n    "sliding midpoint" rule, which ensures that the cells do not all\n    become long and thin.\n\n    The tree can be queried for the r closest neighbors of any given point\n    (optionally returning only those within some maximum distance of the\n    point). It can also be queried, with a substantial gain in efficiency,\n    for the r approximate closest neighbors.\n\n    For large dimensions (20 is already large) do not expect this to run\n    significantly faster than brute force. High-dimensional nearest-neighbor\n    queries are a substantial open problem in computer science.\n\n    Attributes\n    ----------\n    data : ndarray, shape (n,m)\n        The n data points of dimension m to be indexed. This array is\n        not copied unless this is necessary to produce a contiguous\n        array of doubles. The data are also copied if the kd-tree is built\n        with `copy_data=True`.\n    leafsize : positive int\n        The number of points at which the algorithm switches over to\n        brute-force.\n    m : int\n        The dimension of a single data-point.\n    n : int\n        The number of data points.\n    maxes : ndarray, shape (m,)\n        The maximum value in each dimension of the n data points.\n    mins : ndarray, shape (m,)\n        The minimum value in each dimension of the n data points.\n    tree : object, class cKDTreeNode\n        This attribute exposes a Python view of the root node in the cKDTree\n        object. A full Python view of the kd-tree is created dynamically\n        on the first access. This attribute allows you to create your own\n        query functions in Python.\n    size : int\n        The number of nodes in the tree.\n\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce_cython__(self) -> typing.Any:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    def __setstate_cython__(self) -> typing.Any:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def _build_weights(self, weights) -> typing.Any:
        '\n        _build_weights(weights)\n\n        Compute weights of nodes from weights of data points. This will sum\n        up the total weight per node. This function is used internally.\n\n        Parameters\n        ----------\n        weights : array_like\n            weights of data points; must be the same length as the data points.\n            currently only scalar weights are supported. Therefore the weights\n            array must be 1 dimensional.\n\n        Returns\n        -------\n        node_weights : array_like\n            total weight for each KD-Tree node.\n\n        '
        ...
    
    @property
    def boxsize(self) -> typing.Any:
        ...
    
    def count_neighbors(self, other, r, p=..., weights=..., cumulative=...) -> typing.Any:
        '\n        count_neighbors(self, other, r, p=2., weights=None, cumulative=True)\n\n        Count how many nearby pairs can be formed.\n\n        Count the number of pairs ``(x1,x2)`` can be formed, with ``x1`` drawn\n        from ``self`` and ``x2`` drawn from ``other``, and where\n        ``distance(x1, x2, p) <= r``.\n\n        Data points on ``self`` and ``other`` are optionally weighted by the\n        ``weights`` argument. (See below)\n\n        This is adapted from the "two-point correlation" algorithm described by\n        Gray and Moore [1]_.  See notes for further discussion.\n\n        Parameters\n        ----------\n        other : cKDTree instance\n            The other tree to draw points from, can be the same tree as self.\n        r : float or one-dimensional array of floats\n            The radius to produce a count for. Multiple radii are searched with\n            a single tree traversal.\n            If the count is non-cumulative(``cumulative=False``), ``r`` defines\n            the edges of the bins, and must be non-decreasing.\n        p : float, optional\n            1<=p<=infinity.\n            Which Minkowski p-norm to use.\n            Default 2.0.\n            A finite large p may cause a ValueError if overflow can occur.\n        weights : tuple, array_like, or None, optional\n            If None, the pair-counting is unweighted.\n            If given as a tuple, weights[0] is the weights of points in ``self``, and\n            weights[1] is the weights of points in ``other``; either can be None to\n            indicate the points are unweighted.\n            If given as an array_like, weights is the weights of points in ``self``\n            and ``other``. For this to make sense, ``self`` and ``other`` must be the\n            same tree. If ``self`` and ``other`` are two different trees, a ``ValueError``\n            is raised.\n            Default: None\n        cumulative : bool, optional\n            Whether the returned counts are cumulative. When cumulative is set to ``False``\n            the algorithm is optimized to work with a large number of bins (>10) specified\n            by ``r``. When ``cumulative`` is set to True, the algorithm is optimized to work\n            with a small number of ``r``. Default: True\n\n        Returns\n        -------\n        result : scalar or 1-D array\n            The number of pairs. For unweighted counts, the result is integer.\n            For weighted counts, the result is float.\n            If cumulative is False, ``result[i]`` contains the counts with\n            ``(-inf if i == 0 else r[i-1]) < R <= r[i]``\n\n        Notes\n        -----\n        Pair-counting is the basic operation used to calculate the two point\n        correlation functions from a data set composed of position of objects.\n\n        Two point correlation function measures the clustering of objects and\n        is widely used in cosmology to quantify the large scale structure\n        in our Universe, but it may be useful for data analysis in other fields\n        where self-similar assembly of objects also occur.\n\n        The Landy-Szalay estimator for the two point correlation function of\n        ``D`` measures the clustering signal in ``D``. [2]_\n\n        For example, given the position of two sets of objects,\n\n        - objects ``D`` (data) contains the clustering signal, and\n\n        - objects ``R`` (random) that contains no signal,\n\n        .. math::\n\n             \\xi(r) = \\frac{<D, D> - 2 f <D, R> + f^2<R, R>}{f^2<R, R>},\n\n        where the brackets represents counting pairs between two data sets\n        in a finite bin around ``r`` (distance), corresponding to setting\n        `cumulative=False`, and ``f = float(len(D)) / float(len(R))`` is the\n        ratio between number of objects from data and random.\n\n        The algorithm implemented here is loosely based on the dual-tree\n        algorithm described in [1]_. We switch between two different\n        pair-cumulation scheme depending on the setting of ``cumulative``.\n        The computing time of the method we use when for\n        ``cumulative == False`` does not scale with the total number of bins.\n        The algorithm for ``cumulative == True`` scales linearly with the\n        number of bins, though it is slightly faster when only\n        1 or 2 bins are used. [5]_.\n\n        As an extension to the naive pair-counting,\n        weighted pair-counting counts the product of weights instead\n        of number of pairs.\n        Weighted pair-counting is used to estimate marked correlation functions\n        ([3]_, section 2.2),\n        or to properly calculate the average of data per distance bin\n        (e.g. [4]_, section 2.1 on redshift).\n\n        .. [1] Gray and Moore,\n               "N-body problems in statistical learning",\n               Mining the sky, 2000, :arxiv:`astro-ph/0012333`\n\n        .. [2] Landy and Szalay,\n               "Bias and variance of angular correlation functions",\n               The Astrophysical Journal, 1993, :doi:`10.1086/172900`\n\n        .. [3] Sheth, Connolly and Skibba,\n               "Marked correlations in galaxy formation models",\n               2005, :arxiv:`astro-ph/0511773`\n\n        .. [4] Hawkins, et al.,\n               "The 2dF Galaxy Redshift Survey: correlation functions,\n               peculiar velocities and the matter density of the Universe",\n               Monthly Notices of the Royal Astronomical Society, 2002,\n               :doi:`10.1046/j.1365-2966.2003.07063.x`\n\n        .. [5] https://github.com/scipy/scipy/pull/5647#issuecomment-168474926\n\n        Examples\n        --------\n        You can count neighbors number between two kd-trees within a distance:\n\n        >>> import numpy as np\n        >>> from scipy.spatial import cKDTree\n        >>> np.random.seed(21701)\n        >>> points1 = np.random.random((5, 2))\n        >>> points2 = np.random.random((5, 2))\n        >>> kd_tree1 = cKDTree(points1)\n        >>> kd_tree2 = cKDTree(points2)\n        >>> kd_tree1.count_neighbors(kd_tree2, 0.2)\n        9\n\n        This number is same as the total pair number calculated by\n        `query_ball_tree`:\n\n        >>> indexes = kd_tree1.query_ball_tree(kd_tree2, r=0.2)\n        >>> sum([len(i) for i in indexes])\n        9\n\n        '
        ...
    
    @property
    def data(self) -> typing.Any:
        ...
    
    @property
    def indices(self) -> typing.Any:
        ...
    
    @property
    def leafsize(self) -> typing.Any:
        ...
    
    @property
    def m(self) -> typing.Any:
        ...
    
    @property
    def maxes(self) -> typing.Any:
        ...
    
    @property
    def mins(self) -> typing.Any:
        ...
    
    @property
    def n(self) -> typing.Any:
        ...
    
    def query(self, x, k=..., eps=..., p=..., distance_upper_bound=..., workers=...) -> typing.Any:
        '\n        query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf, workers=1)\n\n        Query the kd-tree for nearest neighbors\n\n        Parameters\n        ----------\n        x : array_like, last dimension self.m\n            An array of points to query.\n        k : list of integer or integer\n            The list of k-th nearest neighbors to return. If k is an\n            integer it is treated as a list of [1, ... k] (range(1, k+1)).\n            Note that the counting starts from 1.\n        eps : non-negative float\n            Return approximate nearest neighbors; the k-th returned value\n            is guaranteed to be no further than (1+eps) times the\n            distance to the real k-th nearest neighbor.\n        p : float, 1<=p<=infinity\n            Which Minkowski p-norm to use.\n            1 is the sum-of-absolute-values "Manhattan" distance\n            2 is the usual Euclidean distance\n            infinity is the maximum-coordinate-difference distance\n            A finite large p may cause a ValueError if overflow can occur.\n        distance_upper_bound : nonnegative float\n            Return only neighbors within this distance.  This is used to prune\n            tree searches, so if you are doing a series of nearest-neighbor\n            queries, it may help to supply the distance to the nearest neighbor\n            of the most recent point.\n        workers : int, optional\n            Number of workers to use for parallel processing. If -1 is given\n            all CPU threads are used. Default: 1.\n\n            .. versionchanged:: 1.6.0\n               The "n_jobs" argument was renamed "workers". The old name\n               "n_jobs" is deprecated and will stop working in SciPy 1.8.0.\n\n        Returns\n        -------\n        d : array of floats\n            The distances to the nearest neighbors.\n            If ``x`` has shape ``tuple+(self.m,)``, then ``d`` has shape ``tuple+(k,)``.\n            When k == 1, the last dimension of the output is squeezed.\n            Missing neighbors are indicated with infinite distances.\n        i : ndarray of ints\n            The index of each neighbor in ``self.data``.\n            If ``x`` has shape ``tuple+(self.m,)``, then ``i`` has shape ``tuple+(k,)``.\n            When k == 1, the last dimension of the output is squeezed.\n            Missing neighbors are indicated with ``self.n``.\n\n        Notes\n        -----\n        If the KD-Tree is periodic, the position ``x`` is wrapped into the\n        box.\n\n        When the input k is a list, a query for arange(max(k)) is performed, but\n        only columns that store the requested values of k are preserved. This is\n        implemented in a manner that reduces memory usage.\n\n        Examples\n        --------\n\n        >>> import numpy as np\n        >>> from scipy.spatial import cKDTree\n        >>> x, y = np.mgrid[0:5, 2:8]\n        >>> tree = cKDTree(np.c_[x.ravel(), y.ravel()])\n\n        To query the nearest neighbours and return squeezed result, use\n\n        >>> dd, ii = tree.query([[0, 0], [2.1, 2.9]], k=1)\n        >>> print(dd, ii)\n        [2.         0.14142136] [ 0 13]\n\n        To query the nearest neighbours and return unsqueezed result, use\n\n        >>> dd, ii = tree.query([[0, 0], [2.1, 2.9]], k=[1])\n        >>> print(dd, ii)\n        [[2.        ]\n         [0.14142136]] [[ 0]\n         [13]]\n\n        To query the second nearest neighbours and return unsqueezed result, use\n\n        >>> dd, ii = tree.query([[0, 0], [2.1, 2.9]], k=[2])\n        >>> print(dd, ii)\n        [[2.23606798]\n         [0.90553851]] [[ 6]\n         [12]]\n\n        To query the first and second nearest neighbours, use\n\n        >>> dd, ii = tree.query([[0, 0], [2.1, 2.9]], k=2)\n        >>> print(dd, ii)\n        [[2.         2.23606798]\n         [0.14142136 0.90553851]] [[ 0  6]\n         [13 12]]\n\n        or, be more specific\n\n        >>> dd, ii = tree.query([[0, 0], [2.1, 2.9]], k=[1, 2])\n        >>> print(dd, ii)\n        [[2.         2.23606798]\n         [0.14142136 0.90553851]] [[ 0  6]\n         [13 12]]\n\n        '
        ...
    
    def query_ball_point(self, x, r, p=..., eps=..., workers=..., return_sorted=..., return_length=...) -> typing.Any:
        '\n        query_ball_point(self, x, r, p=2., eps=0, workers=1, return_sorted=None,\n                         return_length=False)\n\n        Find all points within distance r of point(s) x.\n\n        Parameters\n        ----------\n        x : array_like, shape tuple + (self.m,)\n            The point or points to search for neighbors of.\n        r : array_like, float\n            The radius of points to return, shall broadcast to the length of x.\n        p : float, optional\n            Which Minkowski p-norm to use.  Should be in the range [1, inf].\n            A finite large p may cause a ValueError if overflow can occur.\n        eps : nonnegative float, optional\n            Approximate search. Branches of the tree are not explored if their\n            nearest points are further than ``r / (1 + eps)``, and branches are\n            added in bulk if their furthest points are nearer than\n            ``r * (1 + eps)``.\n        workers : int, optional\n            Number of jobs to schedule for parallel processing. If -1 is given\n            all processors are used. Default: 1.\n\n            .. versionchanged:: 1.6.0\n               The "n_jobs" argument was renamed "workers". The old name\n               "n_jobs" is deprecated and will stop working in SciPy 1.8.0.\n\n        return_sorted : bool, optional\n            Sorts returned indicies if True and does not sort them if False. If\n            None, does not sort single point queries, but does sort\n            multi-point queries which was the behavior before this option\n            was added.\n\n            .. versionadded:: 1.2.0\n        return_length: bool, optional\n            Return the number of points inside the radius instead of a list\n            of the indices.\n            .. versionadded:: 1.3.0\n\n        Returns\n        -------\n        results : list or array of lists\n            If `x` is a single point, returns a list of the indices of the\n            neighbors of `x`. If `x` is an array of points, returns an object\n            array of shape tuple containing lists of neighbors.\n\n        Notes\n        -----\n        If you have many points whose neighbors you want to find, you may save\n        substantial amounts of time by putting them in a cKDTree and using\n        query_ball_tree.\n\n        Examples\n        --------\n        >>> from scipy import spatial\n        >>> x, y = np.mgrid[0:4, 0:4]\n        >>> points = np.c_[x.ravel(), y.ravel()]\n        >>> tree = spatial.cKDTree(points)\n        >>> tree.query_ball_point([2, 0], 1)\n        [4, 8, 9, 12]\n\n        Query multiple points and plot the results:\n\n        >>> import matplotlib.pyplot as plt\n        >>> points = np.asarray(points)\n        >>> plt.plot(points[:,0], points[:,1], \'.\')\n        >>> for results in tree.query_ball_point(([2, 0], [3, 3]), 1):\n        ...     nearby_points = points[results]\n        ...     plt.plot(nearby_points[:,0], nearby_points[:,1], \'o\')\n        >>> plt.margins(0.1, 0.1)\n        >>> plt.show()\n\n        '
        ...
    
    def query_ball_tree(self, other, r, p=..., eps=...) -> typing.Any:
        '\n        query_ball_tree(self, other, r, p=2., eps=0)\n\n        Find all pairs of points between `self` and `other` whose distance is at most r\n\n        Parameters\n        ----------\n        other : cKDTree instance\n            The tree containing points to search against.\n        r : float\n            The maximum distance, has to be positive.\n        p : float, optional\n            Which Minkowski norm to use.  `p` has to meet the condition\n            ``1 <= p <= infinity``.\n            A finite large p may cause a ValueError if overflow can occur.\n        eps : float, optional\n            Approximate search.  Branches of the tree are not explored\n            if their nearest points are further than ``r/(1+eps)``, and\n            branches are added in bulk if their furthest points are nearer\n            than ``r * (1+eps)``.  `eps` has to be non-negative.\n\n        Returns\n        -------\n        results : list of lists\n            For each element ``self.data[i]`` of this tree, ``results[i]`` is a\n            list of the indices of its neighbors in ``other.data``.\n\n        Examples\n        --------\n        You can search all pairs of points between two kd-trees within a distance:\n\n        >>> import matplotlib.pyplot as plt\n        >>> import numpy as np\n        >>> from scipy.spatial import cKDTree\n        >>> np.random.seed(21701)\n        >>> points1 = np.random.random((15, 2))\n        >>> points2 = np.random.random((15, 2))\n        >>> plt.figure(figsize=(6, 6))\n        >>> plt.plot(points1[:, 0], points1[:, 1], "xk", markersize=14)\n        >>> plt.plot(points2[:, 0], points2[:, 1], "og", markersize=14)\n        >>> kd_tree1 = cKDTree(points1)\n        >>> kd_tree2 = cKDTree(points2)\n        >>> indexes = kd_tree1.query_ball_tree(kd_tree2, r=0.2)\n        >>> for i in range(len(indexes)):\n        ...     for j in indexes[i]:\n        ...         plt.plot([points1[i, 0], points2[j, 0]],\n        ...             [points1[i, 1], points2[j, 1]], "-r")\n        >>> plt.show()\n\n        '
        ...
    
    def query_pairs(self, r, p=..., eps=...) -> typing.Any:
        '\n        query_pairs(self, r, p=2., eps=0)\n\n        Find all pairs of points in `self` whose distance is at most r.\n\n        Parameters\n        ----------\n        r : positive float\n            The maximum distance.\n        p : float, optional\n            Which Minkowski norm to use.  ``p`` has to meet the condition\n            ``1 <= p <= infinity``.\n            A finite large p may cause a ValueError if overflow can occur.\n        eps : float, optional\n            Approximate search.  Branches of the tree are not explored\n            if their nearest points are further than ``r/(1+eps)``, and\n            branches are added in bulk if their furthest points are nearer\n            than ``r * (1+eps)``.  `eps` has to be non-negative.\n        output_type : string, optional\n            Choose the output container, \'set\' or \'ndarray\'. Default: \'set\'\n\n        Returns\n        -------\n        results : set or ndarray\n            Set of pairs ``(i,j)``, with ``i < j``, for which the corresponding\n            positions are close. If output_type is \'ndarray\', an ndarry is\n            returned instead of a set.\n\n        Examples\n        --------\n        You can search all pairs of points in a kd-tree within a distance:\n\n        >>> import matplotlib.pyplot as plt\n        >>> import numpy as np\n        >>> from scipy.spatial import cKDTree\n        >>> np.random.seed(21701)\n        >>> points = np.random.random((20, 2))\n        >>> plt.figure(figsize=(6, 6))\n        >>> plt.plot(points[:, 0], points[:, 1], "xk", markersize=14)\n        >>> kd_tree = cKDTree(points)\n        >>> pairs = kd_tree.query_pairs(r=0.2)\n        >>> for (i, j) in pairs:\n        ...     plt.plot([points[i, 0], points[j, 0]],\n        ...             [points[i, 1], points[j, 1]], "-r")\n        >>> plt.show()\n\n        '
        ...
    
    @property
    def size(self) -> typing.Any:
        ...
    
    def sparse_distance_matrix(self, other, max_distance, p=...) -> typing.Any:
        '\n        sparse_distance_matrix(self, other, max_distance, p=2.)\n\n        Compute a sparse distance matrix\n\n        Computes a distance matrix between two cKDTrees, leaving as zero\n        any distance greater than max_distance.\n\n        Parameters\n        ----------\n        other : cKDTree\n\n        max_distance : positive float\n\n        p : float, 1<=p<=infinity\n            Which Minkowski p-norm to use.\n            A finite large p may cause a ValueError if overflow can occur.\n\n        output_type : string, optional\n            Which container to use for output data. Options: \'dok_matrix\',\n            \'coo_matrix\', \'dict\', or \'ndarray\'. Default: \'dok_matrix\'.\n\n        Returns\n        -------\n        result : dok_matrix, coo_matrix, dict or ndarray\n            Sparse matrix representing the results in "dictionary of keys"\n            format. If a dict is returned the keys are (i,j) tuples of indices.\n            If output_type is \'ndarray\' a record array with fields \'i\', \'j\',\n            and \'v\' is returned,\n\n        Examples\n        --------\n        You can compute a sparse distance matrix between two kd-trees:\n\n        >>> import numpy as np\n        >>> from scipy.spatial import cKDTree\n        >>> np.random.seed(21701)\n        >>> points1 = np.random.random((5, 2))\n        >>> points2 = np.random.random((5, 2))\n        >>> kd_tree1 = cKDTree(points1)\n        >>> kd_tree2 = cKDTree(points2)\n        >>> sdm = kd_tree1.sparse_distance_matrix(kd_tree2, 0.3)\n        >>> sdm.toarray()\n        array([[0.20220215, 0.14538496, 0.,         0.10257199, 0.        ],\n            [0.13491385, 0.27251306, 0.,         0.18793787, 0.        ],\n            [0.19262396, 0.,         0.,         0.25795122, 0.        ],\n            [0.14859639, 0.07076002, 0.,         0.04065851, 0.        ],\n            [0.17308768, 0.,         0.,         0.24823138, 0.        ]])\n\n        You can check distances above the `max_distance` are zeros:\n\n        >>> from scipy.spatial import distance_matrix\n        >>> distance_matrix(points1, points2)\n        array([[0.20220215, 0.14538496, 0.43588092, 0.10257199, 0.4555495 ],\n            [0.13491385, 0.27251306, 0.65944131, 0.18793787, 0.68184154],\n            [0.19262396, 0.34121593, 0.72176889, 0.25795122, 0.74538858],\n            [0.14859639, 0.07076002, 0.48505773, 0.04065851, 0.50043591],\n            [0.17308768, 0.32837991, 0.72760803, 0.24823138, 0.75017239]])\n\n        '
        ...
    
    @property
    def tree(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class cKDTreeNode(_mod_builtins.object):
    "\n    class cKDTreeNode\n\n    This class exposes a Python view of a node in the cKDTree object.\n\n    All attributes are read-only.\n\n    Attributes\n    ----------\n    level : int\n        The depth of the node. 0 is the level of the root node.\n    split_dim : int\n        The dimension along which this node is split. If this value is -1\n        the node is a leafnode in the kd-tree. Leafnodes are not split further\n        and scanned by brute force.\n    split : float\n        The value used to separate split this node. Points with value >= split\n        in the split_dim dimension are sorted to the 'greater' subnode\n        whereas those with value < split are sorted to the 'lesser' subnode.\n    children : int\n        The number of data points sorted to this node.\n    data_points : ndarray of float64\n        An array with the data points sorted to this node.\n    indices : ndarray of intp\n        An array with the indices of the data points sorted to this node. The\n        indices refer to the position in the data set used to construct the\n        kd-tree.\n    lesser : cKDTreeNode or None\n        Subnode with the 'lesser' data points. This attribute is None for\n        leafnodes.\n    greater : cKDTreeNode or None\n        Subnode with the 'greater' data points. This attribute is None for\n        leafnodes.\n\n    "
    def __init__(self, *args, **kwargs) -> None:
        "\n    class cKDTreeNode\n\n    This class exposes a Python view of a node in the cKDTree object.\n\n    All attributes are read-only.\n\n    Attributes\n    ----------\n    level : int\n        The depth of the node. 0 is the level of the root node.\n    split_dim : int\n        The dimension along which this node is split. If this value is -1\n        the node is a leafnode in the kd-tree. Leafnodes are not split further\n        and scanned by brute force.\n    split : float\n        The value used to separate split this node. Points with value >= split\n        in the split_dim dimension are sorted to the 'greater' subnode\n        whereas those with value < split are sorted to the 'lesser' subnode.\n    children : int\n        The number of data points sorted to this node.\n    data_points : ndarray of float64\n        An array with the data points sorted to this node.\n    indices : ndarray of intp\n        An array with the indices of the data points sorted to this node. The\n        indices refer to the position in the data set used to construct the\n        kd-tree.\n    lesser : cKDTreeNode or None\n        Subnode with the 'lesser' data points. This attribute is None for\n        leafnodes.\n    greater : cKDTreeNode or None\n        Subnode with the 'greater' data points. This attribute is None for\n        leafnodes.\n\n    "
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def children(self) -> typing.Any:
        ...
    
    @property
    def data_points(self) -> typing.Any:
        ...
    
    @property
    def end_idx(self) -> typing.Any:
        ...
    
    @property
    def greater(self) -> typing.Any:
        ...
    
    @property
    def indices(self) -> typing.Any:
        ...
    
    @property
    def lesser(self) -> typing.Any:
        ...
    
    @property
    def level(self) -> typing.Any:
        ...
    
    @property
    def split(self) -> typing.Any:
        ...
    
    @property
    def split_dim(self) -> typing.Any:
        ...
    
    @property
    def start_idx(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class coo_entries(_mod_builtins.object):
    @property
    def __array_interface__(self) -> typing.Any:
        ...
    
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def coo_matrix(self) -> typing.Any:
        ...
    
    def dict(self) -> typing.Any:
        ...
    
    def dok_matrix(self) -> typing.Any:
        ...
    
    def ndarray(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class ordered_pairs(_mod_builtins.object):
    @property
    def __array_interface__(self) -> typing.Any:
        ...
    
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def ndarray(self) -> typing.Any:
        ...
    
    def set(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

def __getattr__(name) -> typing.Any:
    ...




from ..classes.graph import Graph

__all__ = [
    "karate_club_graph",
    "davis_southern_women_graph",
    "florentine_families_graph",
    "les_miserables_graph",
]

def karate_club_graph(): ...
def davis_southern_women_graph(): ...
def florentine_families_graph(): ...
def les_miserables_graph(): ...



from typing import Mapping
from itertools import chain

from ...classes.graph import Graph

__all__ = ["adjacency_data", "adjacency_graph"]

_attrs = ...

def adjacency_data(G: Graph, attrs: Mapping = ...) -> Mapping: ...
def adjacency_graph(
    data: Mapping, directed: bool = False, multigraph: bool = True, attrs: Mapping = ...
): ...



from numpy.typing import NDArray
from typing import Callable, Mapping
import numpy as np

def block_reduce(
    image: NDArray,
    block_size=2,
    func: Callable = ...,
    cval: float = 0,
    func_kwargs: Mapping | None = None,
) -> NDArray: ...



from typing import Any

from django.db.backends.sqlite3.introspection import (
    DatabaseIntrospection as DatabaseIntrospection,
)
from django.db.backends.sqlite3.introspection import (
    FlexibleFieldLookupDict as FlexibleFieldLookupDict,
)

class GeoFlexibleFieldLookupDict(FlexibleFieldLookupDict):
    base_data_types_reverse: Any = ...

class SpatiaLiteIntrospection(DatabaseIntrospection):
    data_types_reverse: Any = ...
    def get_geometry_type(self, table_name: Any, description: Any) -> Any: ...
    def get_constraints(self, cursor: Any, table_name: Any) -> Any: ...




import numpy as np


class SplitInfo:
    """Pure data class to store information about a potential split.
    Parameters
    ----------
    gain : float
        The gain of the split.
    feature_idx : int
        The index of the feature to be split.
    bin_idx : int
        The index of the bin on which the split is made. Should be ignored if
        `is_categorical` is True: `left_cat_bitset` will be used to determine
        the split.
    missing_go_to_left : bool
        Whether missing values should go to the left child. This is used
        whether the split is categorical or not.
    sum_gradient_left : float
        The sum of the gradients of all the samples in the left child.
    sum_hessian_left : float
        The sum of the hessians of all the samples in the left child.
    sum_gradient_right : float
        The sum of the gradients of all the samples in the right child.
    sum_hessian_right : float
        The sum of the hessians of all the samples in the right child.
    n_samples_left : int, default=0
        The number of samples in the left child.
    n_samples_right : int
        The number of samples in the right child.
    is_categorical : bool
        Whether the split is done on a categorical feature.
    left_cat_bitset : ndarray of shape=(8,), dtype=uint32 or None
        Bitset representing the categories that go to the left. This is used
        only when `is_categorical` is True.
        Note that missing values are part of that bitset if there are missing
        values in the training data. For missing values, we rely on that
        bitset for splitting, but at prediction time, we rely on
        missing_go_to_left.
    """
    def __init__(self, gain: float, feature_idx: int, bin_idx: int,
                 missing_go_to_left: bool, sum_gradient_left: float, sum_hessian_left: float,
                 sum_gradient_right: float, sum_hessian_right: float, n_samples_left: int,
                 n_samples_right: int, value_left, value_right,
                 is_categorical: bool, left_cat_bitset: np.ndarray|None=None): ...


class Splitter:
    """Splitter used to find the best possible split at each node.
    A split (see SplitInfo) is characterized by a feature and a bin.
    The Splitter is also responsible for partitioning the samples among the
    leaves of the tree (see split_indices() and the partition attribute).
    Parameters
    ----------
    X_binned : ndarray of int, shape (n_samples, n_features)
        The binned input samples. Must be Fortran-aligned.
    n_bins_non_missing : ndarray, shape (n_features,)
        For each feature, gives the number of bins actually used for
        non-missing values.
    missing_values_bin_idx : uint8
        Index of the bin that is used for missing values. This is the index of
        the last bin and is always equal to max_bins (as passed to the GBDT
        classes), or equivalently to n_bins - 1.
    has_missing_values : ndarray, shape (n_features,)
        Whether missing values were observed in the training data, for each
        feature.
    is_categorical : ndarray of bool of shape (n_features,)
        Indicates categorical features.
    monotonic_cst : ndarray of int of shape (n_features,), dtype=int
        Indicates the monotonic constraint to enforce on each feature.
          - 1: monotonic increase
          - 0: no constraint
          - -1: monotonic decrease
        Read more in the :ref:`User Guide <monotonic_cst_gbdt>`.
    l2_regularization : float
        The L2 regularization parameter.
    min_hessian_to_split : float, default=1e-3
        The minimum sum of hessians needed in each node. Splits that result in
        at least one child having a sum of hessians less than
        min_hessian_to_split are discarded.
    min_samples_leaf : int, default=20
        The minimum number of samples per leaf.
    min_gain_to_split : float, default=0.0
        The minimum gain needed to split a node. Splits with lower gain will
        be ignored.
    hessians_are_constant: bool, default is False
        Whether hessians are constant.
    n_threads : int, default=1
        Number of OpenMP threads to use.
    """
    X_binned: np.ndarray
    n_features: int
    n_bins_non_missing: np.ndarray
    missing_values_bin_idx: int
    has_missing_values: np.ndarray
    is_categorical: np.ndarray
    monotonic_cst: np.ndarray
    hessians_are_constant: int
    l2_regularization: float
    min_hessian_to_split: float
    min_samples_leaf: int
    min_gain_to_split: float

    partition: np.ndarray
    left_indices_buffer: np.ndarray
    right_indices_buffer: np.ndarray
    n_threads: int

    def __init__(self,
                 X_binned: np.ndarray,
                 n_bins_non_missing: np.ndarray,
                 missing_values_bin_idx: int,
                 has_missing_values: np.ndarray,
                 is_categorical: np.ndarray,
                 monotonic_cst: np.ndarray,
                 l2_regularization: float,
                 min_hessian_to_split: float=1e-3,
                 min_samples_leaf: int=20,
                 min_gain_to_split: float=0.,
                 hessians_are_constant: bool=False,
                 n_threads: int = 1):
        ...

    def split_indices(self, split_info: SplitInfo, sample_indices: np.ndarray):
        """Split samples into left and right arrays.
        The split is performed according to the best possible split
        (split_info).
        Ultimately, this is nothing but a partition of the sample_indices
        array with a given pivot, exactly like a quicksort subroutine.
        Parameters
        ----------
        split_info : SplitInfo
            The SplitInfo of the node to split.
        sample_indices : ndarray of unsigned int, shape (n_samples_at_node,)
            The indices of the samples at the node to split. This is a view
            on self.partition, and it is modified inplace by placing the
            indices of the left child at the beginning, and the indices of
            the right child at the end.
        Returns
        -------
        left_indices : ndarray of int, shape (n_left_samples,)
            The indices of the samples in the left child. This is a view on
            self.partition.
        right_indices : ndarray of int, shape (n_right_samples,)
            The indices of the samples in the right child. This is a view on
            self.partition.
        right_child_position : int
            The position of the right child in ``sample_indices``.
        """
        ...

    def find_node_split(
            self,
            n_samples: int,
            histograms,  # IN
            sum_gradients: float,
            sum_hessians: float,
            value: float,
            lower_bound: float=...,
            upper_bound: float=...,
            allowed_features: np.ndarray|None=None,
            ):
        """For each feature, find the best bin to split on at a given node.
        Return the best split info among all features.
        Parameters
        ----------
        n_samples : int
            The number of samples at the node.
        histograms : ndarray of HISTOGRAM_DTYPE of                 shape (n_features, max_bins)
            The histograms of the current node.
        sum_gradients : float
            The sum of the gradients for each sample at the node.
        sum_hessians : float
            The sum of the hessians for each sample at the node.
        value : float
            The bounded value of the current node. We directly pass the value
            instead of re-computing it from sum_gradients and sum_hessians,
            because we need to compute the loss and the gain based on the
            *bounded* value: computing the value from
            sum_gradients / sum_hessians would give the unbounded value, and
            the interaction with min_gain_to_split would not be correct
            anymore. Side note: we can't use the lower_bound / upper_bound
            parameters either because these refer to the bounds of the
            children, not the bounds of the current node.
        lower_bound : float
            Lower bound for the children values for respecting the monotonic
            constraints.
        upper_bound : float
            Upper bound for the children values for respecting the monotonic
            constraints.
        allowed_features : None or ndarray, dtype=np.uint32
            Indices of the features that are allowed by interaction constraints to be
            split.
        Returns
        -------
        best_split_info : SplitInfo
            The info about the best possible split among all features.
        """   
        ...  





"""
This type stub file was generated by pyright.
"""

import os

"""Tools for coloring text in ANSI terminals.
"""
__all__ = ['TermColors', 'InputTermColors', 'ColorScheme', 'ColorSchemeTable']
color_templates = ...
def make_color_table(in_class): # -> None:
    """Build a set of color attributes in a class.

    Helper function for building the :class:`TermColors` and
    :class`InputTermColors`.
    """
    ...

class TermColors:
    """Color escape sequences.

    This class defines the escape sequences for all the standard (ANSI?)
    colors in terminals. Also defines a NoColor escape which is just the null
    string, suitable for defining 'dummy' color schemes in terminals which get
    confused by color escapes.

    This class should be used as a mixin for building color schemes."""
    NoColor = ...
    Normal = ...
    _base = ...


class InputTermColors:
    """Color escape sequences for input prompts.

    This class is similar to TermColors, but the escapes are wrapped in \\001
    and \\002 so that readline can properly know the length of each line and
    can wrap lines accordingly.  Use this class for any colored text which
    needs to be used in input prompts, such as in calls to raw_input().

    This class defines the escape sequences for all the standard (ANSI?)
    colors in terminals. Also defines a NoColor escape which is just the null
    string, suitable for defining 'dummy' color schemes in terminals which get
    confused by color escapes.

    This class should be used as a mixin for building color schemes."""
    NoColor = ...
    if os.name == 'nt' and os.environ.get('TERM', 'dumb') == 'emacs':
        Normal = ...
        _base = ...
    else:
        Normal = ...
        _base = ...


class NoColors:
    """This defines all the same names as the colour classes, but maps them to
    empty strings, so it can easily be substituted to turn off colours."""
    NoColor = ...
    Normal = ...


class ColorScheme:
    """Generic color scheme class. Just a name and a Struct."""
    def __init__(self, __scheme_name_, colordict=..., **colormap) -> None:
        ...
    
    def copy(self, name=...): # -> ColorScheme:
        """Return a full copy of the object, optionally renaming it."""
        ...
    


class ColorSchemeTable(dict):
    """General class to handle tables of color schemes.

    It's basically a dict of color schemes with a couple of shorthand
    attributes and some convenient methods.

    active_scheme_name -> obvious
    active_colors -> actual color table of the active scheme"""
    def __init__(self, scheme_list=..., default_scheme=...) -> None:
        """Create a table of color schemes.

        The table can be created empty and manually filled or it can be
        created with a list of valid color schemes AND the specification for
        the default active scheme.
        """
        ...
    
    def copy(self): # -> ColorSchemeTable:
        """Return full copy of object"""
        ...
    
    def add_scheme(self, new_scheme): # -> None:
        """Add a new color scheme to the table."""
        ...
    
    def set_active_scheme(self, scheme, case_sensitive=...): # -> None:
        """Set the currently active scheme.

        Names are by default compared in a case-insensitive way, but this can
        be changed by setting the parameter case_sensitive to true."""
        ...
    





from vispy.util.svg.shapes import Rect
from vispy.util.event import Event

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .base_camera import BaseCamera
from ...geometry import Rect
from ...visuals.transforms import STTransform, MatrixTransform

DEFAULT_RECT_TUPLE = ...

class PanZoomCamera(BaseCamera):

    _state_props = ...

    def __init__(self, rect: Rect = ..., aspect: None | float = None, **kwargs): ...
    @property
    def aspect(self): ...
    @aspect.setter
    def aspect(self, value): ...
    def zoom(self, factor: float | tuple, center: tuple | None = None): ...
    def pan(self, *pan): ...
    @property
    def rect(self): ...
    @rect.setter
    def rect(self, args): ...
    @property
    def center(self): ...
    @center.setter
    def center(self, center): ...
    def _set_range(self, init): ...
    def viewbox_resize_event(self, event: Event): ...
    def viewbox_mouse_event(self, event: Event): ...
    def _update_transform(self): ...



from typing import Sequence

def dayofweek(y: int, m: int, d: int) -> int: ...
def is_leapyear(year: int) -> bool: ...
def get_days_in_month(year: int, month: int) -> int: ...
def get_week_of_year(year: int, month: int, day: int) -> int: ...

# cpdef iso_calendar_t get_iso_calendar(int year, int month, int day) nogil
def get_day_of_year(year: int, month: int, day: int) -> int: ...
def get_lastbday(year: int, month: int) -> int: ...
def get_firstbday(year: int, month: int) -> int: ...

DAY_NANOS: int = ...
HOUR_NANOS: int = ...
month_offset: Sequence[int] = ...



from numpy.typing import ArrayLike

from ..classes.graph import Graph

__all__ = ["attr_matrix", "attr_sparse_matrix"]

def attr_matrix(
    G: Graph,
    edge_attr: str | None = None,
    node_attr: str | None = None,
    normalized: bool = False,
    rc_order: ArrayLike | None = None,
    dtype=None,
    order=None,
): ...
def attr_sparse_matrix(
    G: Graph,
    edge_attr: str | None = None,
    node_attr: str | None = None,
    normalized: bool = False,
    rc_order: ArrayLike | None = None,
    dtype=None,
): ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""Displayhook for IPython.

This defines a callable class that IPython uses for `sys.displayhook`.
"""
class DisplayHook(Configurable):
    """The custom IPython displayhook to replace sys.displayhook.

    This class does many things, but the basic idea is that it is a callable
    that gets called anytime user code returns a value.
    """
    shell = ...
    exec_result = ...
    cull_fraction = ...
    def __init__(self, shell=..., cache_size=..., **kwargs) -> None:
        ...
    
    @property
    def prompt_count(self):
        ...
    
    def check_for_underscore(self): # -> None:
        """Check if the user has set the '_' variable by hand."""
        ...
    
    def quiet(self):
        """Should we silence the display hook because of ';'?"""
        ...
    
    @staticmethod
    def semicolon_at_end_of_expression(expression): # -> None:
        """Parse Python expression and detects whether last token is ';'"""
        ...
    
    def start_displayhook(self): # -> None:
        """Start the displayhook, initializing resources."""
        ...
    
    def write_output_prompt(self): # -> None:
        """Write the output prompt.

        The default implementation simply writes the prompt to
        ``sys.stdout``.
        """
        ...
    
    def compute_format_data(self, result):
        """Compute format data of the object to be displayed.

        The format data is a generalization of the :func:`repr` of an object.
        In the default implementation the format data is a :class:`dict` of
        key value pair where the keys are valid MIME types and the values
        are JSON'able data structure containing the raw data for that MIME
        type. It is up to frontends to determine pick a MIME to to use and
        display that data in an appropriate manner.

        This method only computes the format data for the object and should
        NOT actually print or write that to a stream.

        Parameters
        ----------
        result : object
            The Python object passed to the display hook, whose format will be
            computed.

        Returns
        -------
        (format_dict, md_dict) : dict
            format_dict is a :class:`dict` whose keys are valid MIME types and values are
            JSON'able raw data for that MIME type. It is recommended that
            all return values of this should always include the "text/plain"
            MIME type representation of the object.
            md_dict is a :class:`dict` with the same MIME type keys
            of metadata associated with each output.

        """
        ...
    
    prompt_end_newline = ...
    def write_format_data(self, format_dict, md_dict=...) -> None:
        """Write the format data dict to the frontend.

        This default version of this method simply writes the plain text
        representation of the object to ``sys.stdout``. Subclasses should
        override this method to send the entire `format_dict` to the
        frontends.

        Parameters
        ----------
        format_dict : dict
            The format dict for the object passed to `sys.displayhook`.
        md_dict : dict (optional)
            The metadata dict to be associated with the display data.
        """
        ...
    
    def update_user_ns(self, result): # -> None:
        """Update user_ns with various things like _, __, _1, etc."""
        ...
    
    def fill_exec_result(self, result): # -> None:
        ...
    
    def log_output(self, format_dict): # -> None:
        """Log the output."""
        ...
    
    def finish_displayhook(self): # -> None:
        """Finish up all displayhook activities."""
        ...
    
    def __call__(self, result=...): # -> None:
        """Printing with history cache management.

        This is invoked every time the interpreter needs to print, and is
        activated by setting the variable sys.displayhook to it.
        """
        ...
    
    def cull_cache(self): # -> None:
        """Output cache is full, cull the oldest entries"""
        ...
    
    def flush(self): # -> None:
        ...
    


class CapturingDisplayHook:
    def __init__(self, shell, outputs=...) -> None:
        ...
    
    def __call__(self, result=...): # -> None:
        ...
    





from typing import Any, Callable, Mapping
from numpy.random import RandomState
from ..utils import check_random_state as check_random_state, check_array as check_array
from ..utils._bunch import Bunch
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import MatrixLike, ArrayLike, Int
from ..metrics import check_scoring as check_scoring
import numbers
import numpy as np


def permutation_importance(
    estimator: Any,
    X: MatrixLike,
    y: None | MatrixLike | ArrayLike,
    *,
    scoring: ArrayLike | None | tuple | Callable | Mapping | str = None,
    n_repeats: Int = 5,
    n_jobs: None | int = None,
    random_state: RandomState | None | Int = None,
    sample_weight: None | ArrayLike = None,
    max_samples: float | int = 1.0,
) -> Bunch | dict[str, Bunch]:
    ...



from typing import ClassVar, Literal, TypeVar
from numpy.random import RandomState
from itertools import chain as chain
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from ..metrics import accuracy_score as accuracy_score, r2_score as r2_score
from ..preprocessing import LabelBinarizer as LabelBinarizer
from ..utils.validation import check_is_fitted as check_is_fitted
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..utils._param_validation import (
    StrOptions as StrOptions,
    Options as Options,
    Interval as Interval,
)
from numbers import Integral as Integral, Real as Real
from ..base import (
    BaseEstimator,
    ClassifierMixin,
    RegressorMixin,
    is_classifier as is_classifier,
)
from ..model_selection import train_test_split as train_test_split
from ..utils import (
    gen_batches as gen_batches,
    check_random_state as check_random_state,
    shuffle,
    column_or_1d as column_or_1d,
)
from ._stochastic_optimizers import (
    SGDOptimizer as SGDOptimizer,
    AdamOptimizer as AdamOptimizer,
)
from ._base import (
    ACTIVATIONS as ACTIVATIONS,
    DERIVATIVES as DERIVATIVES,
    LOSS_FUNCTIONS as LOSS_FUNCTIONS,
)
from ..utils.multiclass import (
    unique_labels as unique_labels,
    type_of_target as type_of_target,
)
from ..utils.metaestimators import available_if as available_if
from .._typing import Float, MatrixLike, ArrayLike, Int

BaseMultilayerPerceptron_Self = TypeVar(
    "BaseMultilayerPerceptron_Self", bound="BaseMultilayerPerceptron"
)
MLPRegressor_Self = TypeVar("MLPRegressor_Self", bound="MLPRegressor")
MLPClassifier_Self = TypeVar("MLPClassifier_Self", bound="MLPClassifier")

import numpy as np
import warnings

import scipy.optimize


_STOCHASTIC_SOLVERS: list = ...


class BaseMultilayerPerceptron(BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        hidden_layer_sizes: tuple[int, int] | tuple[int] | list[int],
        activation: str,
        solver: str,
        alpha: Float,
        batch_size: str,
        learning_rate: str,
        learning_rate_init: float,
        power_t: float,
        max_iter: int,
        loss: str,
        shuffle: bool,
        random_state: None | int,
        tol: float,
        verbose: int | bool,
        warm_start: bool,
        momentum: float | int,
        nesterovs_momentum: bool,
        early_stopping: bool,
        validation_fraction: float,
        beta_1: float,
        beta_2: float,
        epsilon: float,
        n_iter_no_change: int,
        max_fun: int,
    ) -> None:
        ...

    def fit(
        self: BaseMultilayerPerceptron_Self, X: MatrixLike, y: MatrixLike | ArrayLike
    ) -> BaseMultilayerPerceptron_Self:
        ...


class MLPClassifier(ClassifierMixin, BaseMultilayerPerceptron):
    out_activation_: str = ...
    n_outputs_: int = ...
    n_layers_: int = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercepts_: list = ...
    coefs_: list = ...
    t_: int = ...
    best_validation_score_: float | None = ...
    validation_scores_: None | list = ...
    loss_curve_: list = ...
    best_loss_: float | None = ...
    loss_: float = ...
    classes_: ndarray | list[ndarray] = ...

    def __init__(
        self,
        hidden_layer_sizes: ArrayLike | tuple[int] = ...,
        activation: Literal["relu", "identity", "logistic", "tanh", "relu"] = "relu",
        *,
        solver: Literal["lbfgs", "sgd", "adam", "adam"] = "adam",
        alpha: Float = 0.0001,
        batch_size: str | Int = "auto",
        learning_rate: Literal[
            "constant", "invscaling", "adaptive", "constant"
        ] = "constant",
        learning_rate_init: Float = 0.001,
        power_t: Float = 0.5,
        max_iter: Int = 200,
        shuffle: bool = True,
        random_state: RandomState | None | Int = None,
        tol: Float = 1e-4,
        verbose: bool = False,
        warm_start: bool = False,
        momentum: Float = 0.9,
        nesterovs_momentum: bool = True,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        beta_1: Float = 0.9,
        beta_2: Float = 0.999,
        epsilon: Float = 1e-8,
        n_iter_no_change: Int = 10,
        max_fun: Int = 15000,
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def partial_fit(
        self: MLPClassifier_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
    ) -> MLPClassifier_Self:
        ...

    def predict_log_proba(self, X: ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class MLPRegressor(RegressorMixin, BaseMultilayerPerceptron):
    out_activation_: str = ...
    n_outputs_: int = ...
    n_layers_: int = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercepts_: list = ...
    coefs_: list = ...
    t_: int = ...
    best_validation_score_: float | None = ...
    validation_scores_: None | list = ...
    loss_curve_: list = ...
    best_loss_: float = ...
    loss_: float = ...

    def __init__(
        self,
        hidden_layer_sizes: ArrayLike | tuple[int, int] = ...,
        activation: Literal["relu", "identity", "logistic", "tanh", "relu"] = "relu",
        *,
        solver: Literal["lbfgs", "sgd", "adam", "adam"] = "adam",
        alpha: Float = 0.0001,
        batch_size: str | Int = "auto",
        learning_rate: Literal[
            "constant", "invscaling", "adaptive", "constant"
        ] = "constant",
        learning_rate_init: Float = 0.001,
        power_t: Float = 0.5,
        max_iter: Int = 200,
        shuffle: bool = True,
        random_state: RandomState | None | Int = None,
        tol: Float = 1e-4,
        verbose: bool = False,
        warm_start: bool = False,
        momentum: Float = 0.9,
        nesterovs_momentum: bool = True,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        beta_1: Float = 0.9,
        beta_2: Float = 0.999,
        epsilon: Float = 1e-8,
        n_iter_no_change: Int = 10,
        max_fun: Int = 15000,
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def partial_fit(
        self: MLPRegressor_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> MLPRegressor_Self:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._lsq.givens_elimination, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def givens_elimination() -> typing.Any:
    'Zero out a diagonal block of a matrix by series of Givens rotations.\n\n    The matrix has the structure::\n\n        [ S ]\n        [ D ]\n\n    Where S is an upper triangular matrix with shape (n, n) and D is a\n    diagonal matrix with shape (n, n) with elements from `diag`. This function\n    applies Givens rotations to it such that the resulting matrix has zeros\n    in place of D.\n\n    Array `S` will be modified in-place.\n\n    Array `v` of shape (n,) is the part of the full vector with shape (2*n,)::\n\n        [ v ]\n        [ 0 ]\n\n    to which Givens rotations are applied. This array is modified in place,\n    such that on exit it contains the first n components of the above\n    mentioned vector after rotations were applied.\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize.__nnls, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def nnls(a, m, n, b, w, zz, index_bn, maxiter, mda=..., overwrite_a=..., overwrite_b=...) -> typing.Any:
    "x,rnorm,mode = nnls(a,m,n,b,w,zz,index_bn,maxiter,[mda,overwrite_a,overwrite_b])\n\nWrapper for ``nnls``.\n\nParameters\n----------\na : input rank-2 array('d') with bounds (mda,*)\nm : input int\nn : input int\nb : input rank-1 array('d') with bounds (*)\nw : input rank-1 array('d') with bounds (*)\nzz : input rank-1 array('d') with bounds (*)\nindex_bn : input rank-1 array('i') with bounds (*)\nmaxiter : input int\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\nmda : input int, optional\n    Default: shape(a,0)\noverwrite_b : input int, optional\n    Default: 0\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\nrnorm : float\nmode : int\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from typing import Optional, Any
import numpy as np
from numpy.typing import ArrayLike, DTypeLike

def compare_render(
    orig_data: ArrayLike,
    rendered_data: ArrayLike,
    previous_render: Optional[ArrayLike] = None,
    atol: Optional[float] = 1.0,
): ...
def max_for_dtype(input_dtype: DTypeLike): ...
def make_rgba(data_in: ArrayLike) -> ArrayLike: ...




import numpy as np


class HistogramBuilder:
    """A Histogram builder... used to build histograms.
    A histogram is an array with n_bins entries of type HISTOGRAM_DTYPE. Each
    feature has its own histogram. A histogram contains the sum of gradients
    and hessians of all the samples belonging to each bin.
    There are different ways to build a histogram:
    - by subtraction: hist(child) = hist(parent) - hist(sibling)
    - from scratch. In this case we have routines that update the hessians
      or not (not useful when hessians are constant for some losses e.g.
      least squares). Also, there's a special case for the root which
      contains all the samples, leading to some possible optimizations.
      Overall all the implementations look the same, and are optimized for
      cache hit.
    Parameters
    ----------
    X_binned : ndarray of int, shape (n_samples, n_features)
        The binned input samples. Must be Fortran-aligned.
    n_bins : int
        The total number of bins, including the bin for missing values. Used
        to define the shape of the histograms.
    gradients : ndarray, shape (n_samples,)
        The gradients of each training sample. Those are the gradients of the
        loss w.r.t the predictions, evaluated at iteration i - 1.
    hessians : ndarray, shape (n_samples,)
        The hessians of each training sample. Those are the hessians of the
        loss w.r.t the predictions, evaluated at iteration i - 1.
    hessians_are_constant : bool
        Whether hessians are constant.
    """
    def __init__(self, X_binned: np.ndarray,
                 n_bins: int, gradients: np.ndarray,
                 hessians: np.ndarray,
                 hessians_are_constant: bool,
                 n_threads: int) -> None:
        ...

    def compute_histograms_brute(
        self,
        sample_indices: np.ndarray,    
        allowed_features: np.ndarray|None = None,
    ) -> np.ndarray:
        """Compute the histograms of the node by scanning through all the data.
        For a given feature, the complexity is O(n_samples)
        Parameters
        ----------
        sample_indices : array of int, shape (n_samples_at_node,)
            The indices of the samples at the node to split.
        allowed_features : None or ndarray, dtype=np.uint32
            Indices of the features that are allowed by interaction constraints to be
            split.
        Returns
        -------
        histograms : ndarray of HISTOGRAM_DTYPE, shape (n_features, n_bins)
            The computed histograms of the current node.
        """
        ...


    def compute_histograms_subtraction(
        self,
        parent_histograms: np.ndarray,
        sibling_histograms: np.ndarray,
        allowed_features: np.ndarray|None = None,
    ) -> np.ndarray:
        """Compute the histograms of the node using the subtraction trick.
        hist(parent) = hist(left_child) + hist(right_child)
        For a given feature, the complexity is O(n_bins). This is much more
        efficient than compute_histograms_brute, but it's only possible for one
        of the siblings.
        Parameters
        ----------
        parent_histograms : ndarray of HISTOGRAM_DTYPE,                 shape (n_features, n_bins)
            The histograms of the parent.
        sibling_histograms : ndarray of HISTOGRAM_DTYPE,                 shape (n_features, n_bins)
            The histograms of the sibling.
        allowed_features : None or ndarray, dtype=np.uint32
            Indices of the features that are allowed by interaction constraints to be
            split.
        Returns
        -------
        histograms : ndarray of HISTOGRAM_DTYPE, shape(n_features, n_bins)
            The computed histograms of the current node.
        """
        ...





from typing import Any, Callable, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy.sparse.linalg import eigsh as eigsh
from ..preprocessing import KernelCenterer as KernelCenterer
from scipy import linalg as linalg
from ..exceptions import NotFittedError as NotFittedError
from numpy import ndarray
from ..utils.extmath import svd_flip as svd_flip
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from ..metrics.pairwise import pairwise_kernels as pairwise_kernels
from .._typing import Int, Float, MatrixLike, ArrayLike
from ..utils.validation import check_is_fitted as check_is_fitted

KernelPCA_Self = TypeVar("KernelPCA_Self", bound="KernelPCA")


# Author: Mathieu Blondel <mathieu@mblondel.org>
#         Sylvain Marie <sylvain.marie@schneider-electric.com>
# License: BSD 3 clause

import numpy as np


class KernelPCA(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    X_fit_: ndarray = ...
    X_transformed_fit_: ndarray = ...
    dual_coef_: ndarray = ...
    eigenvectors_: ndarray = ...
    eigenvalues_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        kernel: Callable
        | Literal[
            "linear", "poly", "rbf", "sigmoid", "cosine", "precomputed", "linear"
        ] = "linear",
        gamma: None | Float = None,
        degree: Int = 3,
        coef0: Float = 1,
        kernel_params: None | dict = None,
        alpha: Float = 1.0,
        fit_inverse_transform: bool = False,
        eigen_solver: Literal["auto", "dense", "arpack", "randomized", "auto"] = "auto",
        tol: Float = 0,
        max_iter: None | Int = None,
        iterated_power: Literal["auto", "auto"] | int = "auto",
        remove_zero_eig: bool = False,
        random_state: RandomState | None | Int = None,
        copy_X: bool = True,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: KernelPCA_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> KernelPCA_Self:
        ...

    def fit_transform(
        self, X: MatrixLike | ArrayLike, y: Any = None, **params
    ) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...



from numpy.typing import ArrayLike

from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = ["rooted_tree_isomorphism", "tree_isomorphism"]

def root_trees(t1, root1, t2, root2): ...

# figure out the level of each node, with 0 at root
def assign_levels(G: Graph, root): ...

# now group the nodes at each level
def group_by_levels(levels): ...

# now lets get the isomorphism by walking the ordered_children
def generate_isomorphism(v, w, M, ordered_children): ...
def rooted_tree_isomorphism(t1, root1, t2, root2) -> ArrayLike: ...
def tree_isomorphism(t1, t2) -> ArrayLike: ...



from typing import Mapping, Literal
from numpy.typing import ArrayLike
from ...classes.graph import Graph
import math

from ...algorithms.tree.mst import random_spanning_tree
from ...utils import not_implemented_for, pairwise, py_random_state

__all__ = [
    "traveling_salesman_problem",
    "christofides",
    "asadpour_atsp",
    "greedy_tsp",
    "simulated_annealing_tsp",
    "threshold_accepting_tsp",
]

def swap_two_nodes(soln, seed) -> ArrayLike: ...
def move_one_node(soln, seed) -> ArrayLike: ...
def christofides(G: Graph, weight="weight", tree=None) -> ArrayLike: ...
def traveling_salesman_problem(
    G: Graph, weight="weight", nodes=None, cycle=True, method=None
) -> ArrayLike: ...
@py_random_state(2)
def asadpour_atsp(G: Graph, weight="weight", seed=None, source=None): ...
def held_karp_ascent(G: Graph, weight="weight"): ...
def spanning_tree_distribution(G: Graph, z: Mapping) -> Mapping: ...
def greedy_tsp(G: Graph, weight="weight", source=None): ...
@py_random_state(9)
def simulated_annealing_tsp(
    G: Graph,
    init_cycle,
    weight="weight",
    source=None,
    temp=100,
    move="1-1",
    max_iterations=10,
    N_inner=100,
    alpha=0.01,
    seed=None,
): ...
@py_random_state(9)
def threshold_accepting_tsp(
    G: Graph,
    init_cycle: ArrayLike | Literal["greedy"],
    weight="weight",
    source=None,
    threshold=1,
    move="1-1",
    max_iterations=10,
    N_inner=100,
    alpha=0.1,
    seed=None,
): ...



from .collections import PolyCollection
from typing import Literal
from ._typing import *
from .backend_bases import Event, MouseEvent, RendererBase
from .figure import Figure
from .axes import Axes
from .artist import Artist, allow_rasterization

class QuiverKey(Artist):

    halign = ...
    valign = ...
    pivot = ...
    def __init__(
        self,
        Q: Quiver,
        X: float,
        Y: float,
        U: float,
        label: str,
        *,
        angle: float = 0,
        coordinates: Literal["axes", "figure", "data", "inches"] = "axes",
        color: Color = ...,
        labelsep: float = 0.1,
        labelpos: Literal["N", "S", "E", "W"] = ...,
        labelcolor: Color = ...,
        fontproperties: dict = ...,
        **kwargs
    ) -> None: ...
    @property
    def labelsep(self): ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def set_figure(self, fig: Figure): ...
    def contains(self, mouseevent: MouseEvent) -> bool: ...

class Quiver(PolyCollection):
    def __init__(
        self,
        a: Axes,
        *args,
        scale: float = ...,
        headwidth: float = ...,
        headlength: float = ...,
        headaxislength: float = ...,
        minshaft: float = ...,
        minlength: float = ...,
        units=...,
        scale_units=...,
        angles=...,
        width: float = ...,
        color: Color = ...,
        pivot=...,
        **kwargs
    ) -> None: ...
    def get_datalim(self, transData): ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def set_UVC(self, U, V, C=...): ...

    quiver_doc = ...

class Barbs(PolyCollection):
    def __init__(
        self,
        ax: Axes,
        *args,
        pivot=...,
        length: float = ...,
        barbcolor: Color = ...,
        flagcolor: Color = ...,
        sizes=...,
        fill_empty=...,
        barb_increments=...,
        rounding=...,
        flip_barb=...,
        **kwargs
    ) -> None: ...
    def set_UVC(self, U, V, C=...): ...
    def set_offsets(self, xy): ...
    barbs_doc = ...



import numpy as np
from scipy import signal

def approximate_polygon(coords, tolerance: float): ...

# B-Spline subdivision
_SUBDIVISION_MASKS: dict = ...

def subdivide_polygon(coords, degree=2, preserve_ends: bool = False): ...



"""
This type stub file was generated by pyright.
"""

"""
Autocall capabilities for IPython.core.

Authors:

* Brian Granger
* Fernando Perez
* Thomas Kluyver

Notes
-----
"""
class IPyAutocall:
    """ Instances of this class are always autocalled
    
    This happens regardless of 'autocall' variable state. Use this to
    develop macro-like mechanisms.
    """
    _ip = ...
    rewrite = ...
    def __init__(self, ip=...) -> None:
        ...
    
    def set_ip(self, ip): # -> None:
        """Will be used to set _ip point to current ipython instance b/f call

        Override this method if you don't want this to happen.

        """
        ...
    


class ExitAutocall(IPyAutocall):
    """An autocallable object which will be added to the user namespace so that
    exit, exit(), quit or quit() are all valid ways to close the shell."""
    rewrite = ...
    def __call__(self): # -> None:
        ...
    


class ZMQExitAutocall(ExitAutocall):
    """Exit IPython. Autocallable, so it needn't be explicitly called.
    
    Parameters
    ----------
    keep_kernel : bool
      If True, leave the kernel alive. Otherwise, tell the kernel to exit too
      (default).
    """
    def __call__(self, keep_kernel=...): # -> None:
        ...
    





from typing import (
    Hashable,
    Mapping,
    Sequence,
)

from pandas._typing import FilePathOrBuffer

class CSVFormatter:
    obj = ...
    sep = ...
    na_rep = ...
    float_format = ...
    decimal = ...
    header = ...
    index = ...
    index_label = ...
    mode = ...
    encoding = ...
    compression = ...
    quoting = ...
    quotechar = ...
    doublequote = ...
    escapechar = ...
    line_terminator = ...
    date_format = ...
    has_mi_columns = ...
    cols = ...
    blocks = ...
    data = ...
    chunksize = ...
    data_index = ...
    nlevels = ...
    def __init__(
        self,
        obj,
        path_or_buf: FilePathOrBuffer[str] | None = ...,
        sep: str = ...,
        na_rep: str = ...,
        float_format: str | None = ...,
        cols=...,
        header: bool | Sequence[Hashable] = ...,
        index: bool = ...,
        index_label: bool | Hashable | Sequence[Hashable] | None = ...,
        mode: str = ...,
        encoding: str | None = ...,
        compression: str | Mapping[str, str] | None = ...,
        quoting: int | None = ...,
        line_terminator=...,
        chunksize: int | None = ...,
        quotechar=...,
        date_format: str | None = ...,
        doublequote: bool = ...,
        escapechar: str | None = ...,
        decimal=...,
    ) -> None: ...
    writer = ...
    def save(self) -> None: ...



from ._hash import FeatureHasher as FeatureHasher
from .image import img_to_graph as img_to_graph, grid_to_graph as grid_to_graph
from ._dict_vectorizer import DictVectorizer as DictVectorizer
from . import text as text

__all__ = [
    "DictVectorizer",
    "image",
    "img_to_graph",
    "grid_to_graph",
    "text",
    "FeatureHasher",
]



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import os.path as op

# List the vispy fonts made available online
_vispy_fonts = ...

def _get_vispy_font_filename(face, bold, italic): ...



from numpy.distutils.misc_util import Configuration
from numpy.distutils.core import setup
import numpy
import os
import platform


def configuration(parent_package: str = "", top_path=None):
    ...



from ._isomap import Isomap as Isomap
from ._locally_linear import (
    locally_linear_embedding as locally_linear_embedding,
    LocallyLinearEmbedding as LocallyLinearEmbedding,
)
from ._spectral_embedding import (
    SpectralEmbedding as SpectralEmbedding,
    spectral_embedding as spectral_embedding,
)
from ._mds import MDS as MDS, smacof as smacof
from ._t_sne import TSNE as TSNE, trustworthiness as trustworthiness

__all__ = [
    "locally_linear_embedding",
    "LocallyLinearEmbedding",
    "Isomap",
    "MDS",
    "smacof",
    "SpectralEmbedding",
    "spectral_embedding",
    "TSNE",
    "trustworthiness",
]



  
import numpy as np
from ..utils._seq_dataset import SequentialDataset32, SequentialDataset64


def sag32(
    dataset: SequentialDataset32,
    weights_array: np.ndarray,
    intercept_array: np.ndarray,
    n_samples: int,
    n_features: int,
    n_classes: int,
    tol: float,
    max_iter: int,
    loss_function: str,
    step_size: float,
    alpha: float,
    beta: float,
    sum_gradient_init: np.ndarray,
    gradient_memory_init: np.ndarray,
    seen_init: np.ndarray,
    num_seen: int,
    fit_intercept: bool,
    intercept_sum_gradient_init: np.ndarray,
    intercept_decay: float,
    saga: bool,
    verbose: bool
) -> tuple[int, int]: ...


def sag64(
    dataset: SequentialDataset64,
    weights_array: np.ndarray,
    intercept_array: np.ndarray,
    n_samples: int,
    n_features: int,
    n_classes: int,
    tol: float,
    max_iter: int,
    loss_function: str,
    step_size: float,
    alpha: float,
    beta: float,
    sum_gradient_init: np.ndarray,
    gradient_memory_init: np.ndarray,
    seen_init: np.ndarray,
    num_seen: int,
    fit_intercept: bool,
    intercept_sum_gradient_init: np.ndarray,
    intercept_decay: float,
    saga: bool,
    verbose: bool
) -> tuple[int, int]: ...



from typing import Type
from .._config import get_config as get_config
from .._typing import Float
from numpy import ndarray, int32
import numpy
import scipy.special as special


class _ArrayAPIWrapper:
    def __init__(self, array_namespace) -> None:
        ...

    def __getattr__(self, name):
        ...

    def take(self, X, indices, *, axis):
        ...


class _NumPyApiWrapper:
    def __getattr__(self, name: str):
        ...

    def astype(
        self,
        x: ndarray,
        dtype: Type[int32] | Type[int] | Type[Float],
        *,
        copy: bool = True,
        casting: str = "unsafe"
    ) -> ndarray:
        ...

    def asarray(self, x, *, dtype=None, device=None, copy=None) -> ndarray:
        ...

    def unique_inverse(self, x: ndarray) -> tuple[ndarray, ndarray]:
        ...

    def unique_counts(self, x: ndarray) -> tuple[ndarray, ndarray]:
        ...

    def unique_values(self, x: ndarray) -> ndarray:
        ...

    def concat(self, arrays: list[ndarray], *, axis=None) -> ndarray:
        ...


def get_namespace(*arrays) -> tuple[_NumPyApiWrapper, bool]:
    ...



from typing import Mapping
from numpy.typing import ArrayLike
from collections import defaultdict

from ..classes.graph import Graph

__all__ = ["combinatorial_embedding_to_pos"]

def combinatorial_embedding_to_pos(
    embedding, fully_triangulate: bool = False
) -> Mapping: ...
def set_position(parent, tree, remaining_nodes, delta_x, y_coordinate, pos): ...
def get_canonical_ordering(embedding, outer_face: ArrayLike) -> ArrayLike: ...
def triangulate_face(embedding, v1, v2): ...
def triangulate_embedding(embedding, fully_triangulate: bool = True): ...
def make_bi_connected(
    embedding, starting_node, outgoing_node, edges_counted: set
) -> ArrayLike: ...



from .isomorph import (
    could_be_isomorphic as could_be_isomorphic,
    fast_could_be_isomorphic as fast_could_be_isomorphic,
    faster_could_be_isomorphic as faster_could_be_isomorphic,
    is_isomorphic as is_isomorphic,
)
from .vf2userfunc import (
    GraphMatcher as GraphMatcher,
    DiGraphMatcher as DiGraphMatcher,
    MultiGraphMatcher as MultiGraphMatcher,
    MultiDiGraphMatcher as MultiDiGraphMatcher,
)
from .matchhelpers import (
    categorical_node_match as categorical_node_match,
    categorical_edge_match as categorical_edge_match,
    categorical_multiedge_match as categorical_multiedge_match,
    numerical_node_match as numerical_node_match,
    numerical_edge_match as numerical_edge_match,
    numerical_multiedge_match as numerical_multiedge_match,
    generic_node_match as generic_node_match,
    generic_edge_match as generic_edge_match,
    generic_multiedge_match as generic_multiedge_match,
)
from .temporalisomorphvf2 import (
    TimeRespectingGraphMatcher as TimeRespectingGraphMatcher,
    TimeRespectingDiGraphMatcher as TimeRespectingDiGraphMatcher,
)
from .ismags import ISMAGS as ISMAGS
from .tree_isomorphism import (
    rooted_tree_isomorphism as rooted_tree_isomorphism,
    tree_isomorphism as tree_isomorphism,
)



from numpy.typing import NDArray

import numpy as np

from .._shared.utils import deprecate_kwarg

from ._util import (
    _offsets_to_raveled_neighbors,
    _resolve_neighborhood,
    _set_border_values,
)

@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def flood_fill(
    image: NDArray,
    seed_point: tuple | int,
    new_value,
    *,
    footprint: NDArray | None = None,
    connectivity: int | None = None,
    tolerance: float | int | None = None,
    in_place: bool = False
) -> NDArray: ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def flood(
    image: NDArray,
    seed_point: tuple | int,
    *,
    footprint: NDArray | None = None,
    connectivity: int | None = None,
    tolerance: float | int | None = None
) -> NDArray: ...



from django.contrib.gis.db.backends.base.features import (
    BaseSpatialFeatures as BaseSpatialFeatures,
)
from django.db.backends.oracle.features import (
    DatabaseFeatures as OracleDatabaseFeatures,
)

class DatabaseFeatures(BaseSpatialFeatures, OracleDatabaseFeatures):
    supports_add_srs_entry: bool = ...
    supports_geometry_field_introspection: bool = ...
    supports_geometry_field_unique_index: bool = ...
    supports_perimeter_geodetic: bool = ...
    supports_dwithin_distance_expr: bool = ...



from os import makedirs as makedirs, remove as remove
from ._base import RemoteFileMetadata as RemoteFileMetadata, load_descr as load_descr
from os.path import exists as exists
from ..utils import Bunch
from . import get_data_home as get_data_home
import tarfile

import numpy as np
import logging

import joblib


# The original data can be found at:
# https://www.dcc.fc.up.pt/~ltorgo/Regression/cal_housing.tgz
ARCHIVE = ...

logger = ...


def fetch_california_housing(
    *,
    data_home: None | str = None,
    download_if_missing: bool = True,
    return_X_y: bool = False,
    as_frame: bool = False
) -> tuple[Bunch, tuple]:
    ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils.extmath import row_norms as row_norms, stable_cumsum as stable_cumsum
from ..utils.fixes import (
    threadpool_limits as threadpool_limits,
    threadpool_info as threadpool_info,
)
from ..utils.sparsefuncs import mean_variance_axis as mean_variance_axis
from ..utils.sparsefuncs_fast import assign_rows_csr as assign_rows_csr
from ._k_means_common import CHUNK_SIZE as CHUNK_SIZE
from ..metrics.pairwise import euclidean_distances as euclidean_distances
from ..utils.validation import check_is_fitted as check_is_fitted
from ..utils._readonly_array_wrapper import ReadonlyArrayWrapper as ReadonlyArrayWrapper
from abc import ABC, abstractmethod as abstractmethod
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
    validate_params as validate_params,
)
from numbers import Integral as Integral, Real as Real
from ..base import (
    BaseEstimator,
    ClusterMixin,
    TransformerMixin,
    ClassNamePrefixFeaturesOutMixin,
)
from ._k_means_lloyd import (
    lloyd_iter_chunked_dense as lloyd_iter_chunked_dense,
    lloyd_iter_chunked_sparse as lloyd_iter_chunked_sparse,
)
from ..utils import check_array as check_array, check_random_state as check_random_state
from .._typing import MatrixLike, ArrayLike, Int, Float
from ._k_means_elkan import (
    init_bounds_dense as init_bounds_dense,
    init_bounds_sparse as init_bounds_sparse,
    elkan_iter_chunked_dense as elkan_iter_chunked_dense,
    elkan_iter_chunked_sparse as elkan_iter_chunked_sparse,
)

KMeans_Self = TypeVar("KMeans_Self", bound="KMeans")
MiniBatchKMeans_Self = TypeVar("MiniBatchKMeans_Self", bound="MiniBatchKMeans")

import warnings

import numpy as np
import scipy.sparse as sp


###############################################################################
# Initialization heuristic


def kmeans_plusplus(
    X: MatrixLike | ArrayLike,
    n_clusters: Int,
    *,
    x_squared_norms: None | ArrayLike = None,
    random_state: None | RandomState | int = None,
    n_local_trials: None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def k_means(
    X: MatrixLike | ArrayLike,
    n_clusters: Int,
    *,
    sample_weight: None | ArrayLike = None,
    init: MatrixLike
    | Callable
    | Literal["k-means++", "random", "k-means++"] = "k-means++",
    n_init: Literal["auto", "warn"] | int = "warn",
    max_iter: Int = 300,
    verbose: bool = False,
    tol: Float = 1e-4,
    random_state: RandomState | None | Int = None,
    copy_x: bool = True,
    algorithm: Literal["lloyd", "elkan", "auto", "full", "lloyd"] = "lloyd",
    return_n_iter: bool = False,
) -> tuple[ndarray, ndarray, float] | tuple[ndarray, ndarray, float, int]:
    ...


class _BaseKMeans(
    ClassNamePrefixFeaturesOutMixin, TransformerMixin, ClusterMixin, BaseEstimator, ABC
):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: Int,
        *,
        init,
        n_init,
        max_iter,
        tol,
        verbose,
        random_state,
    ) -> None:
        ...

    def fit_predict(
        self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> ndarray:
        ...

    def predict(
        self, X: MatrixLike | ArrayLike, sample_weight: None | ArrayLike = None
    ) -> ndarray:
        ...

    def fit_transform(
        self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def score(
        self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> float:
        ...


class KMeans(_BaseKMeans):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    inertia_: float = ...
    labels_: ndarray = ...
    cluster_centers_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: Int = 8,
        *,
        init: MatrixLike
        | Callable
        | Literal["k-means++", "random", "k-means++"] = "k-means++",
        n_init: Literal["auto", "warn"] | int = "warn",
        max_iter: Int = 300,
        tol: Float = 1e-4,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        copy_x: bool = True,
        algorithm: Literal["lloyd", "elkan", "auto", "full", "lloyd"] = "lloyd",
    ) -> None:
        ...

    def fit(
        self: KMeans_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> KMeans_Self:
        ...


class MiniBatchKMeans(_BaseKMeans):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_steps_: int = ...
    n_iter_: int = ...
    inertia_: float = ...
    labels_: ndarray = ...
    cluster_centers_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: Int = 8,
        *,
        init: MatrixLike
        | Callable
        | Literal["k-means++", "random", "k-means++"] = "k-means++",
        max_iter: Int = 100,
        batch_size: Int = 1024,
        verbose: Int = 0,
        compute_labels: bool = True,
        random_state: RandomState | None | Int = None,
        tol: Float = 0.0,
        max_no_improvement: Int = 10,
        init_size: None | Int = None,
        n_init: Literal["auto", "warn"] | int = "warn",
        reassignment_ratio: Float = 0.01,
    ) -> None:
        ...

    def fit(
        self: MiniBatchKMeans_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> MiniBatchKMeans_Self:
        ...

    def partial_fit(
        self: MiniBatchKMeans_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> MiniBatchKMeans_Self:
        ...



from typing import Any, List, Optional

from django.http.request import HttpRequest
from django.http.response import HttpResponseBase

LEVEL_TAGS: Any

class Message:
    level: int = ...
    message: str = ...
    extra_tags: str = ...
    def __init__(
        self, level: int, message: str, extra_tags: Optional[str] = ...
    ) -> None: ...
    @property
    def tags(self) -> str: ...
    @property
    def level_tag(self) -> str: ...

class BaseStorage:
    request: HttpRequest = ...
    used: bool = ...
    added_new: bool = ...
    def __init__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Any: ...
    def __contains__(self, item: Any) -> Any: ...
    def update(self, response: HttpResponseBase) -> Optional[List[Message]]: ...
    def add(
        self, level: int, message: str, extra_tags: Optional[str] = ...
    ) -> None: ...
    level: Any = ...



from numpy.typing import ArrayLike
import heapq

from ..classes.graph import Graph

__all__ = [
    "is_graphical",
    "is_multigraphical",
    "is_pseudographical",
    "is_digraphical",
    "is_valid_degree_sequence_erdos_gallai",
    "is_valid_degree_sequence_havel_hakimi",
]

def is_graphical(sequence, method="eg") -> bool: ...
def is_valid_degree_sequence_havel_hakimi(deg_sequence: ArrayLike) -> bool: ...
def is_valid_degree_sequence_erdos_gallai(deg_sequence: ArrayLike) -> bool: ...
def is_multigraphical(sequence: ArrayLike) -> bool: ...
def is_pseudographical(sequence) -> bool: ...
def is_digraphical(in_sequence, out_sequence) -> bool: ...



import weakref

from .program import ModularProgram

class MultiProgram(object):
    def __init__(self, vcode="", fcode="", gcode=None): ...
    def add_program(self, name=None): ...
    @property
    def vert(self): ...
    @vert.setter
    def vert(self, code): ...
    @property
    def frag(self): ...
    @frag.setter
    def frag(self, code): ...
    @property
    def geom(self): ...
    @geom.setter
    def geom(self, code): ...
    def __contains__(self, key): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def __iter__(self): ...
    def bind(self, data): ...

class MultiShader(object):
    def __init__(self, program, shader): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def _new_program(self, p): ...
